/* $Id: appointments.c,v 1.27 2004/03/13 15:11:28 mjhammel Exp $ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2002
 *========================================================================
 *              FILE NAME: appointments.c
 *            DESCRIPTION: Apointment manager
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPAppointments, XPApptInitDayAlarm
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log: appointments.c,v $
 * Revision 1.27  2004/03/13 15:11:28  mjhammel
 * Added timers to check if day changed to update hilite in Monthly view of
 * Calendar Manager.
 * Changed version to 3.6.1.
 * Updated History for 3 changes (so far) for 3.6.1.
 *
 * Revision 1.26  2004/03/11 05:44:11  mjhammel
 * Rev'd version numbers.
 * Fixed todos to update note if current entry is deleted.
 * Fixed appointments to save notes to appointment entries.
 *
 * Revision 1.25  2004/02/28 03:58:27  mjhammel
 * Allow Enter key in Summary field to accept Editor changes.
 *
 * Revision 1.24  2004/02/27 20:49:34  mjhammel
 * Changed change in Edit calendar to not reset dialog if edit_link != NULL
 *
 * Revision 1.23  2004/02/27 17:53:25  mjhammel
 * Forced monthly cells to 70 pixels in width to get uniform size.
 *
 * Revision 1.22  2004/02/23 04:36:01  mjhammel
 * Calendar complete for beta 1.
 * Initial documentation updates for 3.6.
 * Fixed bug in FindText.c - accept button should be "gshell.search", not
 * "gshell.accept".
 *
 * Revision 1.21  2004/02/20 04:49:23  mjhammel
 * Added CUPS support for printing.
 * Fixed monthly display in appt's to only highlight date for current month.
 * Changed notebook layout in appt's.
 *
 * Revision 1.20  2004/02/18 06:07:42  mjhammel
 * Changed hilight for monthly page to occur in SetupMonthlyPage().
 * Changed ApptWrite routine to unlink file if link list is empty.
 *
 * Revision 1.19  2004/02/17 14:25:54  mjhammel
 * Added first pass at alarm handling for appointments.
 *
 * Revision 1.18  2004/02/16 16:10:39  mjhammel
 * Appointments now properly displays repeat events in all pages.
 *
 * Revision 1.17  2004/02/13 04:00:03  mjhammel
 * Changed Today handling to new format (ie using CheckToday()).
 *
 * Revision 1.16  2004/02/11 03:59:19  mjhammel
 * Added pixmaps to monthly view of calendar.
 * Fixed today display in calendar to show all entries (but more fixes are
 * needed).
 * Added alarm handling to calendar.
 * Added new entry to calendar and made new and edited entries get saved
 * immediately.
 * Fixed much of the repeat handling in the calendar UI.
 *
 * Revision 1.15  2004/02/07 23:15:09  mjhammel
 * Adding code to AcceptAppt() and handling edit dialog interactions.
 * Changed background image (bg-1.xpm) to be a little darker.
 *
 * Revision 1.14  2004/02/06 04:01:26  mjhammel
 * New calendar icon.  Updated postal icon.
 * Added toggle button handling to calendar entry edit dialog.
 *
 * Revision 1.13  2004/02/05 05:31:24  mjhammel
 * Added toggled callback for repeat forever/until.
 *
 * Revision 1.12  2004/02/04 04:55:27  mjhammel
 * Updated theme to something not so ugly.
 * Added extra components to Appointment Edit dialog.
 *
 * Revision 1.11  2004/02/01 17:09:17  mjhammel
 * Fixed Todos so current entry updates are reflected immediately in the CList.
 * Appointments Editor now correctly handles editing existing entry vs. new
 * entry.
 *
 * Revision 1.10  2004/01/29 14:21:26  mjhammel
 * Cleaned up appointment display.  Added stubs for opening appointment editor.
 *
 * Revision 1.9  2004/01/28 14:16:03  mjhammel
 * Updates to initial calendar code.
 *
 * Revision 1.8  2004/01/23 04:22:41  mjhammel
 * Added WriteAppt and FreeAppt to c_pilot.c
 * Added appointment related defines to XNotesplus.h
 * Added appointment prototypes to prototypes.h
 * Added PopulateCalendar() stub to appointments.c
 * Fixed bug in todos that caused crash when todo entry was deleted.
 *
 * Revision 1.7  2002/05/04 04:20:15  mjhammel
 * Minor updates to GUI.
 *
 * Revision 1.6  2002/04/25 03:52:58  mjhammel
 * Updates to appointment interface.
 *
 * Revision 1.5  2002/04/08 03:07:25  mjhammel
 * More udpates for Appointments feature.  Added USE_APPT option to config.h.
 *
 * Revision 1.4  2002/04/07 20:37:24  mjhammel
 * Appointments (ical replacement) updates.
 *
 * Revision 1.3  2002/04/05 05:36:07  mjhammel
 * Appointments feature updates.  Still a long ways to go...
 *
 * Revision 1.2  2002/04/05 00:07:53  mjhammel
 * Fixed precedence of config file vs command line arguments.
 * Minor updates to appointment feature.
 *
 * Revision 1.1  2002/02/21 16:07:07  mjhammel
 * 3.6 prelims
 *
 *
 *========================================================================*/
#define XP_APPT_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/*
#include "callbacks.h"
#include "interface.h"
#include "support.h"
*/

extern PilotAppt_T *Pilot_Appt;


/* === external routines === */


/* === Public routine prototypes === */
int	XPApptInitDayCheck();


/* === Private routine prototypes === */
static void CreateNewApptDialog();
static void AddNewEntry();
static void CloseAppt();
static void AcceptAppt();
static void DeleteAppt();
static void CloseNewAppt();
static void ApptSelectDay();
static void SetUpMonthlyPage();
static void PageChange();
static void PopulateCalendar();
static void ChooseWeekDay();
static void ChooseMonthDay();
static void ChooseToday();
static void RepeatUntilSetup();
static void AlarmSetup();
static void ApptCalCb();
static void ApptNoDateCb();
static void EditDate();
static void RepeatOptionsSetup();
static void RepeatWeekdaySetup();
static void StartHour();
static void StartMin();
static void StartAMPM();
static void EndHour();
static void EndMin();
static void EndAMPM();
static void AlarmAdvType();
static void ResetEditDialog();
static int CheckToday();
static void EditNote();
static void EditNoteCancel();
static void EditNoteSave();
static void ConvertToXNote();
static void EraseNote();
static void ImportXNotes();
static void ImportXNotesCB();
static void AKeyPress();
static int DayCheckOne();
static int ApptDayCheck();


/* === Global Variables === */
extern AppRes			app_res;
extern GtkWidget		*toplevel;


/* === Static Variables === */
static GtkWidget *calendar = NULL;
static GtkWidget *calendar_week = NULL;
static GtkWidget *todaylabel = NULL;
static GtkWidget *weeklabel = NULL;
static GtkWidget *monthlabel = NULL;
static GtkWidget *weeklists[7];
static GtkWidget *appt_widget = NULL;
static GtkWidget *newapptdialog = NULL;
static GtkWidget *monthlynotebook = NULL;
static GtkWidget *todaylist = NULL;
static GtkWidget *textnote = NULL;
static GtkWidget *monthlytable;
static GtkWidget *monthlists[43];
static GtkWidget *editnotewidget;

static GtkWidget *cal_entry_text;
static GtkWidget *entry_calendar;
static GtkWidget *entry_label;
static GtkWidget *repeat_options_radiobutton[5];
static GtkWidget *repeat_forever_togglebutton;
static GtkWidget *repeat_until_togglebutton;
static GtkWidget *repeat_entry_text;
static GtkWidget *repeat_cal_button;
static GtkWidget *start_hour_optionmenu;
static GtkWidget *start_minute_optionmenu;
static GtkWidget *start_ampm_optionmenu;
static GtkWidget *end_hour_optionmenu;
static GtkWidget *end_minute_optionmenu;
static GtkWidget *end_ampm_optionmenu;
static GtkWidget *repeat_weekday_checkbutton[7];
static GtkWidget *alarm_on_radiobutton;
static GtkWidget *alarm_off_radiobutton;
static GtkObject *alarm_adv_adj;
static GtkWidget *alarm_advamount_spinbox;
static GtkWidget *alarm_type_optionmenu;
static GtkWidget *daily_hbox;
static GtkWidget *repeat_forever_hbox;
static GtkWidget *newappt_delete_button;
static GtkWidget *editnote_button;

GdkPixmap *cal_morning_xbm;
GdkPixmap *cal_noon_xbm;
GdkPixmap *cal_night_xbm;
GdkBitmap *cal_morning_mask = NULL;
GdkBitmap *cal_noon_mask = NULL;
GdkBitmap *cal_night_mask = NULL;

static GdkColor		hi_color;
static GdkColor		orig_color;
static GdkColormap	*colormap;

static Calendar_T		ApptCalendar;

static int first_day_offset;
static int calendar_sig_id;
static int calendar_week_sig_id;
static int calendar_entry_sig_id;
static int rpf_sig_id;
static int rpu_sig_id;
static int repeat_options_sig_id[5];
static int repeat_weekday_sig_id[7];
static int repeat_options_set;
static int repeat_weekday_set[7];

static int start_hour;
static int start_min;
static int start_ampm;
static int end_hour;
static int end_min;
static int end_ampm;
static int alarm_state;
static int alarm_type;

static PilotAppt_T	*edit_link = NULL;

static int			pageid;
static GDate		*currentdate = NULL;
static GDate		*absolutedate = NULL;

static int	minute_intervals[4] = { 0, 15, 30, 45 };

static char *months[12] = { 
	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sept", "Oct", "Nov", "Dec"
};
static char *weekdays[7] = { 
	"Sun", "Mon", "Tue", "Wed", "Thur", "Fri", "Sat"
};
static char *fullweekdays[7] = { 
	"Sunday", "Monday", "Tueday", "Wednesday", "Thursday", "Friday", "Saturday"
};

static char *repeat_options_names[5] = {
	XP_S_NONE,
	XP_S_DAILY,
	XP_S_WEEKLY,
	XP_S_MONTHLY,
	XP_S_YEARLY
};

static char *repeat_weekday_names[7] = {
	XP_S_SUN,
	XP_S_MON,
	XP_S_TUE,
	XP_S_WED,
	XP_S_THU,
	XP_S_FRI,
	XP_S_SAT
};


/*========================================================================
 *	Name:			XPAppointments
 *	Prototype:	XPAppointments()
 *					
 *	Description:
 *	Create the Appointments dialog.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPAppointments (void)
{
#ifdef DEBUG
	char	fname[]="XPAppointments";
#endif

	GtkWidget	*table;
	GtkWidget	*label;
	GtkWidget	*vbox;
	GtkWidget	*scrolledwindow;
	GtkWidget	*hseparator;
	GDate			*gdate;

	char			buf[128];
	int			i, j;
	time_t		time_tp;
	struct tm	*tm_tp;
	int			row, col;
	char			row_data[1][5] = {""};
	char			*rows[1];
	int 			first_day_offset;

	MonthlyCell_T	*linkptr;
	GtkWidget		*close_button;
	GtkWidget		*add_button;
	GtkStyle			*style;
	GShell_T			gshell;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( appt_widget != NULL )
	{
		PopulateCalendar();
		gtk_widget_show(appt_widget);
		return;
	}

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	appt_widget = XPCreateGenericShell(ApptDialog, 
							ACCEPT_BUTTON | CLOSE_BUTTON, &gshell, FALSE );

	vbox = gshell.vbox;
	close_button = gshell.cancel;
	add_button = gshell.accept;

	/* Change the Accept button to be an Add Entry button */
	gtk_label_set_text(GTK_LABEL(GTK_BIN(add_button)->child), XP_S_ADD_ENTRY);

	gtk_signal_connect (GTK_OBJECT (appt_widget), "destroy",
			GTK_SIGNAL_FUNC (CloseAppt), NULL);
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			GTK_SIGNAL_FUNC (CloseAppt), NULL);
	gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
			GTK_SIGNAL_FUNC (AddNewEntry), NULL);

	/*
	 * Generate the colors we'll use for the current day in the
	 * Monthly and Weekly views.
	 */
	colormap = gdk_window_get_colormap (toplevel->window);
	hi_color.red = (guint16)(65535.0);
	hi_color.green = (guint16)(65535.0);
	hi_color.blue = (guint16)(0);
	gdk_color_alloc (colormap, &hi_color);
	orig_color.red = (guint16)(.99*65535.0);
	orig_color.green = (guint16)(0.85*65535.0);
	orig_color.blue = (guint16)(0.69*65535.0);
	gdk_color_alloc (colormap, &orig_color);

	/*
	 * Building the dialog:  Everything goes inside a notebook.
	 */
	monthlynotebook = gtk_notebook_new ();
	gtk_widget_set_name (monthlynotebook, "notebook");
	gtk_widget_show (monthlynotebook);
	gtk_container_add (GTK_CONTAINER (vbox), monthlynotebook);
	gtk_signal_connect (GTK_OBJECT (monthlynotebook), "switch-page",
			GTK_SIGNAL_FUNC (PageChange), NULL);

	/* 
	 * A table inside which we'll set the current days schedule.
	 */
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (monthlynotebook), vbox);


	/* The label across the top of the dialog */
	time(&time_tp);
	tm_tp = localtime(&time_tp);
	sprintf(buf, "%s %s %s %d, %d", 
		XP_S_APPT_SUMMARY,
		weekdays[tm_tp->tm_wday], months[tm_tp->tm_mon], 
		tm_tp->tm_mday, tm_tp->tm_year+1900);
	todaylabel = gtk_label_new (buf);
	gtk_widget_show (todaylabel);
	gtk_box_pack_start (GTK_BOX (vbox), todaylabel, FALSE, TRUE, 5);

	hseparator = gtk_hseparator_new ();
	gtk_widget_show (hseparator);
	gtk_box_pack_start (GTK_BOX (vbox), hseparator, FALSE, TRUE, 1);

	/* This is the absolute date - it never changes from the real "today" */
	absolutedate = 
		g_date_new_dmy( tm_tp->tm_mday, tm_tp->tm_mon+1, tm_tp->tm_year+1900);

	/*
	 * Today's appointments get a scrolled window.
	 */
	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_set_name (scrolledwindow, "notebook");
	gtk_widget_show (scrolledwindow);
	gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
	gtk_widget_set_usize (scrolledwindow, 250, -1);

	/*
	 * Add a clist for individual events.
	 * Column 1 gives the start time.
	 * Column 2 gives the end time.
	 * Column 3 is the event.
	 */
	todaylist = gtk_clist_new(3);
	gtk_widget_show (todaylist);
	gtk_container_add (GTK_CONTAINER (scrolledwindow), todaylist);
	gtk_clist_set_column_width(GTK_CLIST(todaylist), 0, 35);
	gtk_clist_set_column_width(GTK_CLIST(todaylist), 1, 35);
	gtk_clist_set_sort_column(GTK_CLIST(todaylist), 0);
	gtk_signal_connect (GTK_OBJECT (todaylist), "select_row",
			GTK_SIGNAL_FUNC (ChooseToday), (gpointer)NULL);

	/* A calendar to choose a date from */
	calendar = gtk_calendar_new ();
	gtk_widget_show (calendar);
	gtk_widget_set_name (calendar, "addresses");
	gtk_box_pack_start (GTK_BOX (vbox), calendar, FALSE, TRUE, 0);
	gtk_calendar_display_options (GTK_CALENDAR (calendar),
		GTK_CALENDAR_SHOW_HEADING | GTK_CALENDAR_SHOW_DAY_NAMES);
	gtk_calendar_select_day(GTK_CALENDAR(calendar), tm_tp->tm_mday);

	/* Initialize the current date */
	currentdate = 
		g_date_new_dmy( tm_tp->tm_mday, tm_tp->tm_mon+1, tm_tp->tm_year+1900);

	calendar_sig_id = gtk_signal_connect (GTK_OBJECT (calendar), "day_selected",
		GTK_SIGNAL_FUNC (ApptSelectDay), (gpointer)1);

	/* The label in the Notebook tab for page 1 */
	label = gtk_label_new ("Today");
	gtk_widget_set_name (label, "label");
	gtk_widget_show (label);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (monthlynotebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (monthlynotebook), 0), label);

	/*
	 * Week at a Glance page.
	 */
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (monthlynotebook), vbox);

	/* The label across the top of the dialog */
	time(&time_tp);
	tm_tp = localtime(&time_tp);
	gdate = 
		g_date_new_dmy( tm_tp->tm_mday, tm_tp->tm_mon+1, tm_tp->tm_year+1900);
	first_day_offset = g_date_weekday(gdate);
	g_date_free(gdate);
	sprintf(buf, "%s %s %d, %d", 
		XP_S_APPT_WEEK_SUMMARY,
		months[tm_tp->tm_mon], 
		tm_tp->tm_mday-first_day_offset, 
		tm_tp->tm_year+1900);
	weeklabel = gtk_label_new (buf);
	gtk_widget_show (weeklabel);
	gtk_box_pack_start (GTK_BOX (vbox), weeklabel, FALSE, TRUE, 5);

	hseparator = gtk_hseparator_new ();
	gtk_widget_show (hseparator);
	gtk_box_pack_start (GTK_BOX (vbox), hseparator, FALSE, TRUE, 1);

	table = gtk_table_new (2, 4, TRUE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);

	calendar_week = gtk_calendar_new ();
	gtk_widget_set_name (calendar_week, "addresses");
	gtk_widget_show (calendar_week);
	gtk_box_pack_start (GTK_BOX (vbox), calendar_week, FALSE, FALSE, 0);
	gtk_calendar_display_options (GTK_CALENDAR (calendar_week),
		GTK_CALENDAR_SHOW_HEADING | GTK_CALENDAR_SHOW_DAY_NAMES);
	gtk_calendar_select_day(GTK_CALENDAR(calendar_week), tm_tp->tm_mday);
	calendar_week_sig_id = 
		gtk_signal_connect (GTK_OBJECT (calendar_week), "day_selected",
		GTK_SIGNAL_FUNC (ApptSelectDay), (gpointer)2);

	/*
	 * Labels and lists for events.
	 */
	for (i=0; i<4; i++)
	{
		vbox = gtk_vbox_new (FALSE, 0);
		gtk_widget_show (vbox);
		gtk_table_attach (GTK_TABLE (table), vbox, i, i+1, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
			0, 0);

		label = gtk_label_new (fullweekdays[i]);
		gtk_widget_set_name (label, "label");
		gtk_widget_show (label);
		gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);

		scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
		gtk_widget_show (scrolledwindow);
		gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

		weeklists[i] = gtk_list_new ();
		gtk_widget_set_name (weeklists[i], "list");
		gtk_widget_show (weeklists[i]);
		gtk_list_set_selection_mode (GTK_LIST (weeklists[i]), 
			GTK_SELECTION_SINGLE);
		gtk_signal_connect (GTK_OBJECT (weeklists[i]), "selection_changed",
			GTK_SIGNAL_FUNC (ChooseWeekDay), weeklists[i]);
		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwindow),
			weeklists[i]);
	}

	j=4;
	for (i=0; i<3; i++)
	{
		vbox = gtk_vbox_new (FALSE, 0);
		gtk_widget_show (vbox);
		gtk_table_attach (GTK_TABLE (table), vbox, i, i+1, 1, 2,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
			0, 0);

		label = gtk_label_new (fullweekdays[j]);
		gtk_widget_set_name (label, "label");
		gtk_widget_show (label);
		gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
		gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);

		scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
		gtk_widget_show (scrolledwindow);
		gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

		weeklists[j] = gtk_list_new ();
		gtk_widget_set_name (weeklists[j], "list");
		gtk_widget_show (weeklists[j]);
		gtk_signal_connect (GTK_OBJECT (weeklists[j]), "selection_changed",
			GTK_SIGNAL_FUNC (ChooseWeekDay), weeklists[j]);
		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwindow),
			weeklists[j]);

		j++;
	}

	label = gtk_label_new ("Week at a glance...");
	gtk_widget_show (label);
	gtk_widget_set_name (label, "label");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (monthlynotebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (monthlynotebook), 1), label);

	/*
	 * The Month at a Glance page.
	 */
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (monthlynotebook), vbox);

	/* The label across the top of the dialog */
	time(&time_tp);
	tm_tp = localtime(&time_tp);
	sprintf(buf, "%s %s %d", 
		XP_S_APPT_MONTH_SUMMARY,
		months[tm_tp->tm_mon], 
		tm_tp->tm_year+1900);
	monthlabel = gtk_label_new (buf);
	gtk_widget_show (monthlabel);
	gtk_box_pack_start (GTK_BOX (vbox), monthlabel, FALSE, TRUE, 5);

	hseparator = gtk_hseparator_new ();
	gtk_widget_show (hseparator);
	gtk_box_pack_start (GTK_BOX (vbox), hseparator, FALSE, TRUE, 1);


	/*
	 * We got the current month earlier.  Use it to set up the page.
	 */

	/*
	 * One big table inside all date entries go.
	 */
	monthlytable = gtk_table_new (7, 7, FALSE);
	gtk_widget_show (monthlytable);
	gtk_box_pack_start (GTK_BOX (vbox), monthlytable, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (monthlytable), 1);
	gtk_table_set_col_spacings (GTK_TABLE (monthlytable), 1);

	/* The day of the week labels. */
	for ( i=0; i<7; i++ )
	{
		label = gtk_label_new (weekdays[i]);
		gtk_widget_show (label);
		gtk_table_attach (GTK_TABLE (monthlytable), label, i, i+1, 0, 1,
			(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
		gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	}

	/*
	 * The CLists in which we'll stuff info later - don't show these yet.  
	 * We'll figure out which ones to show in SetupMonthlyPage.
	 */
	row = 1;
	col = 0;
	rows[0] = row_data[0];
	for ( i=0; i<42; i++ )
	{
		monthlists[i] = gtk_clist_new(1);
		gtk_table_attach (GTK_TABLE (monthlytable), 
			monthlists[i], col, col+1, row, row+1,
			(GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 
			(GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0);

		/* Each clist gets one label and three blank cells to start */
		gtk_clist_append(GTK_CLIST(monthlists[i]), rows);
		gtk_clist_append(GTK_CLIST(monthlists[i]), rows);
		gtk_clist_append(GTK_CLIST(monthlists[i]), rows);
		gtk_clist_append(GTK_CLIST(monthlists[i]), rows);

		gtk_clist_set_selectable (GTK_CLIST (monthlists[i]), 0, FALSE);
		gtk_clist_set_background (GTK_CLIST (monthlists[i]), 0, &orig_color);
		gtk_widget_set_usize(monthlists[i], 70, -1);
		for(j=1; j<4; j++)
		{
			linkptr = (MonthlyCell_T *)malloc(sizeof(MonthlyCell_T));
			bzero((char *)linkptr, sizeof(MonthlyCell_T));
			gtk_clist_set_row_data(GTK_CLIST(monthlists[i]), j, (gpointer)linkptr);
			gtk_clist_set_selectable (GTK_CLIST (monthlists[i]), j, FALSE);
			gtk_clist_set_background (GTK_CLIST (monthlists[i]), j, &orig_color);
		}

		gtk_clist_set_row_height (GTK_CLIST (monthlists[i]), 13);
		gtk_container_set_border_width(GTK_CONTAINER(monthlists[i]), 0);

		/* Each row calls the same callback function. */
		gtk_signal_connect (GTK_OBJECT (monthlists[i]), "select_row",
			GTK_SIGNAL_FUNC (ChooseMonthDay), monthlists[i]);

		/* Go to next column */
		col++;
		if ( col == 7 )
		{
			col = 0;
			row++;
		}
	}

	/* Now draw the monthly page */
	SetUpMonthlyPage(tm_tp->tm_mon, tm_tp->tm_year+1900);

	label = gtk_label_new ("Month at a glance...");
	gtk_widget_show (label);
	gtk_widget_set_name (label, "label");
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (monthlynotebook), 
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (monthlynotebook), 2), label);

	/* While we're at it, lets generate a few bitmaps */
	style = gtk_widget_get_style( appt_widget );
	sprintf(buf, "%s/%s", app_res.xpm_dir, XP_CAL_MORNING_XPM);
	cal_morning_xbm = gdk_pixmap_create_from_xpm(
			toplevel->window,
			&cal_morning_mask,
			&style->bg[GTK_STATE_NORMAL],
			buf);
	sprintf(buf, "%s/%s", app_res.xpm_dir, XP_CAL_NOON_XPM);
	cal_noon_xbm = gdk_pixmap_create_from_xpm(
			toplevel->window,
			&cal_noon_mask,
			&style->bg[GTK_STATE_NORMAL],
			buf);
	sprintf(buf, "%s/%s", app_res.xpm_dir, XP_CAL_NIGHT_XPM);
	cal_night_xbm = gdk_pixmap_create_from_xpm(
			toplevel->window,
			&cal_night_mask,
			&style->bg[GTK_STATE_NORMAL],
			buf);


	/*
	 * Populate the calendar.
 	 */
	PopulateCalendar();

	/*
	 * Show the calendar.
	 */
	gtk_widget_show(appt_widget);
}

/*========================================================================
 *	Name:			SetUpMonthlyPage
 *	Prototype:	SetUpMonthlyPage()
 *					
 *	Description:
 *	Build the Monthly display page in the notebook.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
SetUpMonthlyPage(
	int			month,
	int			year
)
{
	int					row, col;
	int					i, j;
	int					offset;
	int					date;
	GDate					*gdate;
	char					titles[5];
	gboolean				nomore;
	int					currentmonth;

	/* Clear the current clists */
	for(i=0;i<42;i++) 
	{
		for(j=0;j<4;j++) 
		{
			gtk_clist_set_text(GTK_CLIST(monthlists[i]), j, 0, "");
			gtk_clist_set_selectable (GTK_CLIST (monthlists[i]), j, FALSE);
			gtk_clist_set_selectable (GTK_CLIST (monthlists[i]), j, FALSE);
			gtk_clist_set_background (GTK_CLIST (monthlists[i]), j, &orig_color);
		}
		gtk_widget_hide (monthlists[i]);
	}

	/*
	 * Find out what day of the week the month starts on.
	 */
	gdate = g_date_new_dmy( 1, month+1, year);
	first_day_offset = g_date_weekday(gdate);
	if ( first_day_offset == 7 )
		first_day_offset = 0;
	g_date_free(gdate);

	/* What month are we currently in? */
	currentmonth = g_date_month(absolutedate)-1;

	/*
	 * Each day gets a CList of events 
	 */
	row = 1;
	col = 0;
	date = 1;
	offset = 0;
	nomore = FALSE;
	for ( i=0; i<42; i++ )
	{
		if ( ( offset < first_day_offset ) || ( nomore ) )
		{
			offset++;

			/* Go to next column */
			col++;
			if ( col == 7 )
			{
				col = 0;
				row++;
			}

			continue;
		}

		/* The date title */
		sprintf(titles, "%d", date);

		gdate = g_date_new_dmy( date, month+1, year);
		if ( g_date_is_last_of_month(gdate) )
			nomore = TRUE;
		g_date_free(gdate);

		/* The numeric date in each day */
		gtk_widget_show (monthlists[i]);
		gtk_clist_set_text(GTK_CLIST(monthlists[i]), 0, 0, titles);

		/* Highlight today's entry */
		if ( ( date == g_date_day(absolutedate) ) &&
			  ( month == currentmonth ) )
		{
			for (j=0; j<4; j++)
				gtk_clist_set_background (GTK_CLIST (monthlists[i]), j, &hi_color);
		}

		offset++;
		date++;

		/* Go to next column */
		col++;
		if ( col == 7 )
		{
			col = 0;
			row++;
		}
	}

	/*
	 * Show the calendar.
	 */
	gtk_widget_show(monthlytable);
}

/*========================================================================
 *	Name:			CreateNewApptDialog
 *	Prototype:	CreateNewApptDialog()
 *					
 *	Description:
 *	Opens a dialog for adding a new appointment.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
static void
CreateNewApptDialog()
{
#ifdef DEBUG
	char	*fname="CreateNewApptDialog";
#endif

	GtkWidget			*vbox;
	GtkWidget			*vbox2;
	GtkWidget			*vbox3;
	GtkWidget			*hbox;
	GtkWidget			*close_button;
	GtkWidget			*accept_button;
	GtkWidget			*table1;
	GtkWidget			*table2;
	GtkWidget			*label;
	GtkWidget			*hseparator;
	GtkWidget			*menu;
	GtkWidget			*option_menuitem;
	GtkWidget			*frame;
	GtkWidget			*cal_pixmap;
	GdkPixmap			*cal_xbm;
	GdkBitmap			*cal_mask = NULL;
	GtkWidget			*notes_pixmap;
	GdkPixmap			*notes_xbm;
	GdkBitmap			*notes_mask = NULL;
	GtkStyle				*style;

	GShell_T				gshell;
	time_t				time_tp;
	struct tm			*tm_tp;
	char					buf[128];
	int					i;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( newapptdialog != NULL )
	{
		gtk_widget_show(newapptdialog);
		return;
	}

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	newapptdialog = XPCreateGenericShell(NewApptDialog, 
							ACCEPT_BUTTON | CLOSE_BUTTON | OVERWRITE_BUTTON, 
								&gshell, FALSE );

	vbox = gshell.vbox;
	close_button = gshell.cancel;
	accept_button = gshell.accept;
	newappt_delete_button = gshell.overwrite;

	/* Change the Overwrite button to be a Delete Entry button */
	gtk_label_set_text(GTK_LABEL(GTK_BIN(newappt_delete_button)->child),
		XP_S_DELETE_ENTRY);

	gtk_signal_connect (GTK_OBJECT (newapptdialog), "destroy",
			GTK_SIGNAL_FUNC (CloseNewAppt), NULL);
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			GTK_SIGNAL_FUNC (CloseNewAppt), NULL);
	gtk_signal_connect (GTK_OBJECT (accept_button), "clicked",
			GTK_SIGNAL_FUNC (AcceptAppt), NULL);
	gtk_signal_connect (GTK_OBJECT (newappt_delete_button), "clicked",
			GTK_SIGNAL_FUNC (DeleteAppt), NULL);

	/*
	 * A Table for new appointment components.
	 */
	table1 = gtk_table_new (4, 2, FALSE);
	gtk_widget_set_name (table1, "notebook");
	gtk_widget_show (table1);
	gtk_container_add (GTK_CONTAINER (vbox), table1);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 3);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 6);

	/* The label across the top of the dialog */
	time(&time_tp);
	tm_tp = localtime(&time_tp);
	sprintf(buf, "%s %s %s %d, %d", 
		XP_S_APPT_SUMMARY,
		weekdays[tm_tp->tm_wday], months[tm_tp->tm_mon], 
		tm_tp->tm_mday, tm_tp->tm_year+1900);
	entry_label = gtk_label_new (buf);
	gtk_widget_show (entry_label);
	gtk_table_attach (GTK_TABLE (table1), entry_label, 0, 2, 0, 1,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);


	/* A vertical box for the left side of the dialog. */
	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox2);
	gtk_table_attach (GTK_TABLE (table1), vbox2, 0, 1, 1, 2,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);

	/* A calendar to choose a date from */
	entry_calendar = gtk_calendar_new ();
	gtk_widget_show (entry_calendar);
	gtk_widget_set_name (entry_calendar, "addresses");
	gtk_box_pack_start (GTK_BOX (vbox2), entry_calendar, TRUE, TRUE, 0);
	gtk_calendar_display_options (GTK_CALENDAR (entry_calendar),
		GTK_CALENDAR_SHOW_HEADING | GTK_CALENDAR_SHOW_DAY_NAMES);
	gtk_calendar_select_day(GTK_CALENDAR(entry_calendar), tm_tp->tm_mday);

	calendar_entry_sig_id = 
		gtk_signal_connect (GTK_OBJECT (entry_calendar), "day_selected",
			GTK_SIGNAL_FUNC (ApptSelectDay), (gpointer)3);

	/*
	 * A table for the Start/Stop times for a new appointment.
	 */
	table2 = gtk_table_new (3, 3, TRUE);
	gtk_widget_set_name (table2, "notebook");
	gtk_widget_show (table2);
	gtk_container_add (GTK_CONTAINER (vbox2), table2);

	/* Start / Stop time label */
	label = gtk_label_new (XP_S_STARTSTOP);
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table2), label, 0, 3, 0, 1,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

	/*
	 * The Start time for a new appointment.
	 */
	start_hour_optionmenu = gtk_option_menu_new();
	gtk_widget_set_name (start_hour_optionmenu, "optionmenu");
	gtk_widget_show (start_hour_optionmenu);
	gtk_table_attach (GTK_TABLE (table2), start_hour_optionmenu, 0, 1, 1, 2,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	menu = gtk_menu_new ();

	option_menuitem = gtk_menu_item_new_with_label ("12");
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);

	/* Hours to start appointment */
	for (i=1; i<12; i++)
	{
		sprintf(buf, "%d", i);
		option_menuitem = gtk_menu_item_new_with_label (buf);
  		gtk_widget_show (option_menuitem);
  		gtk_menu_append (GTK_MENU (menu), option_menuitem);
		gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (StartHour), (gpointer)i);
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (start_hour_optionmenu), menu);

	/* 15 minute intervals for each hour */
	start_minute_optionmenu = gtk_option_menu_new ();
	gtk_widget_set_name (start_minute_optionmenu, "optionmenu");
	gtk_widget_show (start_minute_optionmenu);
	gtk_table_attach (GTK_TABLE (table2), start_minute_optionmenu, 1, 2, 1, 2,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	menu = gtk_menu_new ();

	for (i=0; i<4; i++)
	{
		sprintf(buf, ":%02d", minute_intervals[i]);
		option_menuitem = gtk_menu_item_new_with_label (buf);
  		gtk_widget_show (option_menuitem);
  		gtk_menu_append (GTK_MENU (menu), option_menuitem);
		gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (StartMin), (gpointer)i);
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (start_minute_optionmenu), menu);

	/* Start time AM / PM menu */
	start_ampm_optionmenu = gtk_option_menu_new ();
	gtk_widget_set_name (start_ampm_optionmenu, "optionmenu");
	gtk_widget_show (start_ampm_optionmenu);
	gtk_table_attach (GTK_TABLE (table2), start_ampm_optionmenu, 2, 3, 1, 2,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	menu = gtk_menu_new ();
	option_menuitem = gtk_menu_item_new_with_label ("AM");
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);
	gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (StartAMPM), (gpointer)0);
	option_menuitem = gtk_menu_item_new_with_label ("PM");
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (start_ampm_optionmenu), menu);
	gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (StartAMPM), (gpointer)1);

	/*
	 * The Stop time for a new appointment.
	 */
	end_hour_optionmenu = gtk_option_menu_new ();
	gtk_widget_set_name (end_hour_optionmenu, "optionmenu");
	gtk_widget_show (end_hour_optionmenu);
	gtk_table_attach (GTK_TABLE (table2), end_hour_optionmenu, 0, 1, 2, 3,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	menu = gtk_menu_new ();

	option_menuitem = gtk_menu_item_new_with_label ("12");
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);

	/* Hours to stop appointment */
	for (i=1; i<12; i++)
	{
		sprintf(buf, "%d", i);
		option_menuitem = gtk_menu_item_new_with_label (buf);
		gtk_widget_show (option_menuitem);
		gtk_menu_append (GTK_MENU (menu), option_menuitem);
		gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (EndHour), (gpointer)i);
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (end_hour_optionmenu), menu);

	/* 15 minute intervals */
	end_minute_optionmenu = gtk_option_menu_new ();
	gtk_widget_set_name (end_minute_optionmenu, "optionmenu");
	gtk_widget_show (end_minute_optionmenu);
	gtk_table_attach (GTK_TABLE (table2), end_minute_optionmenu, 1, 2, 2, 3,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	menu = gtk_menu_new ();

	for (i=0; i<4; i++)
	{
		sprintf(buf, ":%02d", minute_intervals[i]);
		option_menuitem = gtk_menu_item_new_with_label (buf);
  		gtk_widget_show (option_menuitem);
  		gtk_menu_append (GTK_MENU (menu), option_menuitem);
		gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (EndMin), (gpointer)i);
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (end_minute_optionmenu), menu);

	/* AM / PM menu */
	end_ampm_optionmenu = gtk_option_menu_new ();
	gtk_widget_set_name (end_ampm_optionmenu, "optionmenu");
	gtk_widget_show (end_ampm_optionmenu);
	gtk_table_attach (GTK_TABLE (table2), end_ampm_optionmenu, 2, 3, 2, 3,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	menu = gtk_menu_new ();
	option_menuitem = gtk_menu_item_new_with_label ("AM");
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);
	gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (EndAMPM), (gpointer)0);
	option_menuitem = gtk_menu_item_new_with_label ("PM");
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (end_ampm_optionmenu), menu);
	gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (EndAMPM), (gpointer)1);


	/* A vertical box for the right side of the dialog. */
	vbox2 = gtk_vbox_new (FALSE, 5);
	gtk_widget_show (vbox2);
	gtk_table_attach (GTK_TABLE (table1), vbox2, 1, 2, 1, 2,
		(GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 
		(GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0);

	/* Periodic Repeat Options */
	frame = gtk_frame_new("Periodic Repeat Options");
	gtk_widget_show (frame);
	gtk_container_add (GTK_CONTAINER (vbox2), frame);

	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox3);
	gtk_container_add (GTK_CONTAINER (frame), vbox3);

	hbox = gtk_hbox_new (TRUE, 0);
	gtk_widget_show (hbox);
	gtk_container_add (GTK_CONTAINER (vbox3), hbox);

	repeat_options_radiobutton[0] = 
			gtk_radio_button_new_with_label (NULL, repeat_options_names[0]);
	gtk_widget_show (repeat_options_radiobutton[0]);
	gtk_box_pack_start (GTK_BOX (hbox), repeat_options_radiobutton[0], 
			TRUE, TRUE, 0);
	repeat_options_sig_id[0] = 
			gtk_signal_connect (GTK_OBJECT (repeat_options_radiobutton[0]), 
				"toggled", GTK_SIGNAL_FUNC (RepeatOptionsSetup), (gpointer)0);

	for (i=1; i<5; i++)
	{
		repeat_options_radiobutton[i] = 
			gtk_radio_button_new_with_label (
				 gtk_radio_button_group (
					GTK_RADIO_BUTTON (repeat_options_radiobutton[0])), 
				repeat_options_names[i]);
		gtk_widget_show (repeat_options_radiobutton[i]);
		gtk_box_pack_start (GTK_BOX (hbox), repeat_options_radiobutton[i], 
			TRUE, TRUE, 0);
		repeat_options_sig_id[i] = 
			gtk_signal_connect (GTK_OBJECT (repeat_options_radiobutton[i]), 
				"toggled", GTK_SIGNAL_FUNC (RepeatOptionsSetup), (gpointer)i);
	}

	gtk_toggle_button_set_active (
			GTK_TOGGLE_BUTTON (repeat_options_radiobutton[0]), TRUE);

	hseparator = gtk_hseparator_new ();
	gtk_widget_show (hseparator);
	gtk_container_add (GTK_CONTAINER (vbox3), hseparator);

	/* Weekly Repeat day */
	daily_hbox = gtk_hbox_new (TRUE, 0);
	gtk_widget_show (daily_hbox);
	gtk_container_add (GTK_CONTAINER (vbox3), daily_hbox);

	for (i=0; i<7; i++)
	{
		repeat_weekday_checkbutton[i] = 
			gtk_check_button_new_with_label (repeat_weekday_names[i]);
		gtk_widget_show (repeat_weekday_checkbutton[i]);
		gtk_box_pack_start (GTK_BOX (daily_hbox), 
			repeat_weekday_checkbutton[i], FALSE, FALSE, 0);
		repeat_weekday_sig_id[i] = 
			gtk_signal_connect (GTK_OBJECT (repeat_weekday_checkbutton[i]), 
				"toggled", GTK_SIGNAL_FUNC (RepeatWeekdaySetup), (gpointer)i);
	}

	gtk_widget_set_sensitive(GTK_WIDGET(daily_hbox), FALSE);

	/* Repeat Until / Forever */
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox);
	gtk_container_add (GTK_CONTAINER (vbox2), hbox);

	label = gtk_label_new (XP_S_REPEAT_UNTIL);
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	repeat_forever_hbox = gtk_hbox_new (FALSE, 6);
	gtk_widget_show (repeat_forever_hbox);
	gtk_container_add (GTK_CONTAINER (vbox2), repeat_forever_hbox);

	repeat_forever_togglebutton = gtk_check_button_new_with_label (XP_S_FOREVER);
	gtk_widget_show (repeat_forever_togglebutton);
	gtk_box_pack_start (GTK_BOX (repeat_forever_hbox), 
		repeat_forever_togglebutton, FALSE, FALSE, 0);
	rpf_sig_id = 
		gtk_signal_connect (GTK_OBJECT (repeat_forever_togglebutton), "toggled",
			GTK_SIGNAL_FUNC (RepeatUntilSetup), (gpointer)1);

	repeat_until_togglebutton = gtk_check_button_new_with_label (XP_S_UNTIL);
	gtk_widget_show (repeat_until_togglebutton);
	gtk_box_pack_start (GTK_BOX (repeat_forever_hbox), 
		repeat_until_togglebutton, FALSE, FALSE, 0);
	rpu_sig_id = 
		gtk_signal_connect (GTK_OBJECT (repeat_until_togglebutton), "toggled",
			GTK_SIGNAL_FUNC (RepeatUntilSetup), (gpointer)2);

	repeat_entry_text = gtk_entry_new();
	gtk_widget_show(repeat_entry_text);
	gtk_box_pack_start (GTK_BOX (repeat_forever_hbox), 
		repeat_entry_text, TRUE, TRUE, 4);
	gtk_editable_set_editable(GTK_EDITABLE(repeat_entry_text), FALSE);

	repeat_cal_button = gtk_button_new();
	gtk_widget_show(repeat_cal_button);
	gtk_box_pack_start (GTK_BOX (repeat_forever_hbox), 
		repeat_cal_button, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (repeat_cal_button), "clicked",
			GTK_SIGNAL_FUNC (EditDate), (gpointer)0);

	style = gtk_widget_get_style( repeat_cal_button );
	sprintf(buf, "%s/%s", app_res.xpm_dir, XP_CALENDAR_XPM);
	cal_xbm = gdk_pixmap_create_from_xpm(
			toplevel->window,
			&cal_mask,
			&style->bg[GTK_STATE_NORMAL],
			buf);

	cal_pixmap = gtk_pixmap_new(cal_xbm, cal_mask);
	gtk_container_add(GTK_CONTAINER(repeat_cal_button), cal_pixmap);
	gtk_widget_show( cal_pixmap );

	gtk_widget_set_sensitive(GTK_WIDGET(repeat_entry_text), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(repeat_cal_button), FALSE);

	gtk_widget_set_sensitive(GTK_WIDGET(repeat_forever_hbox), FALSE);

	hseparator = gtk_hseparator_new ();
	gtk_widget_show (hseparator);
	gtk_container_add (GTK_CONTAINER (vbox2), hseparator);


	/* Alarm setting */
	hbox = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox);
	gtk_container_add (GTK_CONTAINER (vbox2), hbox);
	label = gtk_label_new (XP_OPTIONS_ALARM);
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	alarm_off_radiobutton = gtk_radio_button_new_with_label
			(NULL, XP_S_APPT_ALARM_OFF);
	gtk_widget_show (alarm_off_radiobutton);
	gtk_box_pack_start (GTK_BOX (hbox), alarm_off_radiobutton, FALSE, FALSE, 0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (alarm_off_radiobutton), TRUE);
	gtk_signal_connect (GTK_OBJECT (alarm_off_radiobutton), "toggled",
			GTK_SIGNAL_FUNC (AlarmSetup), (gpointer)0);

	alarm_on_radiobutton = gtk_radio_button_new_with_label 
			(gtk_radio_button_group (GTK_RADIO_BUTTON (alarm_off_radiobutton)), 
			XP_S_APPT_ALARM_ON);
	gtk_widget_show (alarm_on_radiobutton);
	gtk_box_pack_start (GTK_BOX (hbox), alarm_on_radiobutton, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (alarm_on_radiobutton), "toggled",
			GTK_SIGNAL_FUNC (AlarmSetup), (gpointer)1);

	label = gtk_label_new (XP_S_APPT_NOTIFY);
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	alarm_adv_adj = gtk_adjustment_new(0,0,60.0,1.0,10.0,0);
	alarm_advamount_spinbox =
			gtk_spin_button_new(GTK_ADJUSTMENT(alarm_adv_adj), 1.0, 0);
	gtk_widget_show (alarm_advamount_spinbox);
	gtk_box_pack_start (GTK_BOX (hbox), alarm_advamount_spinbox, FALSE, FALSE,0);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(alarm_adv_adj), 0);

	alarm_type_optionmenu = gtk_option_menu_new ();
	gtk_widget_set_name (alarm_type_optionmenu, "optionmenu");
	gtk_widget_show (alarm_type_optionmenu);
	gtk_box_pack_start (GTK_BOX (hbox), alarm_type_optionmenu, FALSE, FALSE, 0);
	menu = gtk_menu_new ();

	option_menuitem = gtk_menu_item_new_with_label (XP_S_MIN);
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);
	gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (AlarmAdvType), (gpointer)0);

	option_menuitem = gtk_menu_item_new_with_label (XP_S_HOUR);
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);
	gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (AlarmAdvType), (gpointer)1);

	option_menuitem = gtk_menu_item_new_with_label (XP_S_DAYS);
	gtk_widget_show (option_menuitem);
	gtk_menu_append (GTK_MENU (menu), option_menuitem);
	gtk_signal_connect_object (GTK_OBJECT (option_menuitem), 
				"activate", GTK_SIGNAL_FUNC (AlarmAdvType), (gpointer)2);

	gtk_option_menu_set_menu (GTK_OPTION_MENU (alarm_type_optionmenu), menu);

	gtk_widget_set_sensitive(GTK_WIDGET(alarm_advamount_spinbox), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(alarm_type_optionmenu), FALSE);

	/* Field for entering the calendar item text */
	label = gtk_label_new (XP_S_CAL_ITEM);
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 2, 3,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);

	/* Summary text */
	cal_entry_text = gtk_entry_new();
	gtk_widget_show(cal_entry_text);
	gtk_table_attach (GTK_TABLE (table1), cal_entry_text, 1, 3, 3, 4,
		(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_signal_connect(GTK_OBJECT(cal_entry_text), "key_press_event",
		GTK_SIGNAL_FUNC (AKeyPress), (gpointer)NULL);

	/* The Event Notes section */
	hbox = gtk_hbox_new (FALSE, 20);
	gtk_widget_show (hbox);
	gtk_table_attach (GTK_TABLE (table1), hbox, 0, 1, 3, 4,
		(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

	label = gtk_label_new (XP_S_EVENT_NOTES);
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

	editnote_button = gtk_button_new();
	gtk_widget_show (editnote_button);
	gtk_box_pack_start (GTK_BOX (hbox), editnote_button, TRUE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (editnote_button), "clicked",
			GTK_SIGNAL_FUNC (EditNote), (gpointer)0);

	style = gtk_widget_get_style( editnote_button );
	sprintf(buf, "%s/%s", app_res.xpm_dir, XP_NOTES_XPM);
	notes_xbm = gdk_pixmap_create_from_xpm(
			toplevel->window,
			&notes_mask,
			&style->bg[GTK_STATE_NORMAL],
			buf);

	notes_pixmap = gtk_pixmap_new(notes_xbm, notes_mask);
	gtk_container_add(GTK_CONTAINER(editnote_button), notes_pixmap);
	gtk_widget_show( notes_pixmap );
}


/*========================================================================
 *	Name:			AddNewEntry()
 *	Prototype:	AddNewEntry()
 *					
 *	Description:
 *	Open the EntryDialog with no fields set.
 *========================================================================*/
static void
AddNewEntry()
{
#ifdef DEBUG
	char	*fname="AddNewEntry";
#endif
	int		yearnum, monthnum, daynum;

	DBGEnter();

	/* Show the dialog */
	if ( newapptdialog == NULL )
		CreateNewApptDialog();

	/* Hide the delete button - don't need it for new entries. */
	gtk_widget_hide(newappt_delete_button);

	gtk_entry_set_text(GTK_ENTRY(cal_entry_text), "");
	EraseNote();
	gtk_calendar_get_date(GTK_CALENDAR(calendar_week),
				&(yearnum),
				&(monthnum),
				&(daynum)
				);
	gtk_calendar_select_day(GTK_CALENDAR(entry_calendar), daynum);
	gtk_calendar_select_month(GTK_CALENDAR(entry_calendar), monthnum, yearnum);
	ResetEditDialog();

	gtk_widget_show(newapptdialog);
}

/*========================================================================
 *	Name:			ResetEditDialog()
 *	Prototype:	ResetEditDialog()
 *					
 *	Description:
 *	Reset the edit appointment dialog.
 *========================================================================*/
static void
ResetEditDialog()
{
	int		yearnum, monthnum, daynum;
	char		buf[128];
	GDate		*newdate;

	if ( edit_link != NULL )
		return;

	/* Reset our edit dialog for new entries. */
	start_hour = 8;
	start_min = 0;
	start_ampm = 0;
	end_hour = 9;
	end_min = 0;
	end_ampm = 0;
	alarm_state = 0;
	alarm_type = 0;

	gtk_option_menu_set_history (GTK_OPTION_MENU (start_hour_optionmenu), 8);
	gtk_option_menu_set_history (GTK_OPTION_MENU (start_minute_optionmenu), 0);
	gtk_option_menu_set_history (GTK_OPTION_MENU (start_ampm_optionmenu), 0);
	gtk_option_menu_set_history (GTK_OPTION_MENU (end_hour_optionmenu), 9);
	gtk_option_menu_set_history (GTK_OPTION_MENU (end_minute_optionmenu), 0);
	gtk_option_menu_set_history (GTK_OPTION_MENU (end_ampm_optionmenu), 0);

	gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(repeat_options_radiobutton[0]), True);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(repeat_forever_togglebutton),
			FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(repeat_until_togglebutton),
			FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(alarm_advamount_spinbox), 0.0);
	gtk_option_menu_set_history (GTK_OPTION_MENU (alarm_type_optionmenu), 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alarm_off_radiobutton), True);

	gtk_calendar_get_date(GTK_CALENDAR(entry_calendar),
				&(yearnum),
				&(monthnum),
				&(daynum)
				);
	newdate = g_date_new_dmy( daynum, monthnum+1, yearnum);
	sprintf(buf, "%s %s %s %d, %d", 
			XP_S_APPT_SUMMARY, 
			weekdays[g_date_weekday(newdate)], 
			months[monthnum], daynum, yearnum);
	gtk_label_set_text(GTK_LABEL(entry_label), buf);
	free(newdate);
}

/*========================================================================
 *	Name:			StartHour(), StartMin(), StartAMPM()
 *	Description:
 *	Keep track of menu selections in the Edit dialog.
 *========================================================================*/
static void
StartHour(
	int			id
)
{
	start_hour = id;
}
static void
StartMin(
	int			id
)
{
	start_min = id;
}
static void
StartAMPM(
	int			id
)
{
	start_ampm = id;
}

/*========================================================================
 *	Name:			EndHour(), EndMin(), EndAMPM()
 *	Description:
 *	Keep track of menu selections in the Edit dialog.
 *========================================================================*/
static void
EndHour(
	int			id
)
{
	end_hour = id;
}
static void
EndMin(
	int			id
)
{
	end_min = id;
}
static void
EndAMPM(
	int			id
)
{
	end_ampm = id;
}


/*========================================================================
 *	Name:			AlarmSetup()
 *	Prototype:	AlarmSetup()
 *					
 *	Description:
 *	When the Alarm options changes, change the sensitivity of the "advance
 # notice" options.
 *========================================================================*/
static void
AlarmSetup(
	GtkWidget	*widget,
	int			id
)
{
#ifdef DEBUG
	char	*fname="AlarmSetup";
#endif

	DBGEnter();

	if ( id == 0 )
	{
		gtk_widget_set_sensitive(GTK_WIDGET(alarm_advamount_spinbox), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(alarm_type_optionmenu), FALSE);
		gtk_option_menu_set_history (GTK_OPTION_MENU (alarm_type_optionmenu),0);
		gtk_adjustment_set_value(GTK_ADJUSTMENT(alarm_adv_adj), 0);
		alarm_state = 0;
	}
	else
	{
		gtk_widget_set_sensitive(GTK_WIDGET(alarm_advamount_spinbox), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(alarm_type_optionmenu), TRUE);
		alarm_state = 1;
	}
}

/*========================================================================
 *	Name:			AlarmAdvType()
 *	Prototype:	AlarmAdvType()
 *					
 *	Description:
 *	Keep track of which type of units we use for our Advance notify.
 *========================================================================*/
static void
AlarmAdvType(
	int			id
)
{
#ifdef DEBUG
	char	*fname="AlarmAdvType";
#endif

	DBGEnter();
	alarm_type = id;
}


/*========================================================================
 *	Name:			RepeatOptionsSetup()
 *	Prototype:	RepeatOptionsSetup()
 *					
 *	Description:
 *	When the Periodic Repeat options change, enabled or disable the
 * appropriate options.
 *========================================================================*/
static void
RepeatOptionsSetup(
	GtkWidget	*widget,
	int			id
)
{
#ifdef DEBUG
	char	*fname="RepeatOptionsSetup";
#endif

	int		i;
	char		buf[128];
	GDate		*newdate;
	int		yearnum, monthnum, daynum;

	DBGEnter();

	/* Keep track of which options are set. */
	repeat_options_set = id;

	switch ( id )
	{
		case 0:
		case 1:
		case 3:
		case 4:
			for(i=0; i<7; i++)
				gtk_signal_handler_block (
					GTK_OBJECT (repeat_weekday_checkbutton[i]), 
						repeat_weekday_sig_id[i]);
			for(i=0; i<7; i++)
				gtk_toggle_button_set_active(
					GTK_TOGGLE_BUTTON(repeat_weekday_checkbutton[i]), False);
			for(i=0; i<7; i++)
				gtk_signal_handler_unblock (
					GTK_OBJECT (repeat_weekday_checkbutton[i]), 
						repeat_weekday_sig_id[i]);
			gtk_widget_set_sensitive(GTK_WIDGET(daily_hbox), False);
			break;

		case 2:
			gtk_widget_set_sensitive(GTK_WIDGET(daily_hbox), True);
			break;
	}
	switch( id )
	{
		case 0:
			gtk_signal_handler_block (GTK_OBJECT (repeat_forever_togglebutton), 
				rpf_sig_id);
			gtk_signal_handler_block (GTK_OBJECT (repeat_until_togglebutton), 
				rpu_sig_id);
			gtk_toggle_button_set_active(
					GTK_TOGGLE_BUTTON(repeat_forever_togglebutton), False);
			gtk_toggle_button_set_active(
					GTK_TOGGLE_BUTTON(repeat_until_togglebutton), False);
			gtk_signal_handler_unblock (GTK_OBJECT (repeat_forever_togglebutton), 
				rpf_sig_id);
			gtk_signal_handler_unblock (GTK_OBJECT (repeat_until_togglebutton), 
				rpu_sig_id);
			gtk_widget_set_sensitive(GTK_WIDGET(repeat_forever_hbox), False);
			gtk_calendar_get_date(GTK_CALENDAR(entry_calendar),
						&(yearnum),
						&(monthnum),
						&(daynum)
						);
			newdate = g_date_new_dmy(daynum, monthnum+1, yearnum);
			ApptCalendar.daynum = daynum;
			ApptCalendar.monthnum = monthnum;
			ApptCalendar.yearnum = yearnum;
			ApptCalendar.wdnum = g_date_weekday(newdate);
			g_date_free(newdate);
			sprintf(buf, "%s %s %d, %d", 
				weekdays[ApptCalendar.wdnum],
				months[ApptCalendar.monthnum-1],
				ApptCalendar.daynum,
				ApptCalendar.yearnum);
			gtk_entry_set_text(GTK_ENTRY(repeat_entry_text), buf);
			gtk_widget_set_sensitive(GTK_WIDGET(repeat_entry_text), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(repeat_cal_button), FALSE);

			for (i=0; i<7; i++) repeat_weekday_set[i] = 0;
			break;

		default:
			if ( edit_link )
			{
				if ( ! edit_link->repeatForever )
				{
					if ( edit_link->repeatEnd.tm_mday == 0 )
						edit_link->repeatForever = 1;
				}
				else
				{
					if ( edit_link->repeatEnd.tm_mday == 0 )
					{
						edit_link->repeatEnd.tm_mon = g_date_month(currentdate);
						edit_link->repeatEnd.tm_mday = g_date_day(currentdate);
						edit_link->repeatEnd.tm_year = g_date_year(currentdate)-1900;
						edit_link->repeatEnd.tm_wday = g_date_weekday(currentdate);
					}
					sprintf(buf, "%s %s %d, %d", 
						weekdays[edit_link->repeatEnd.tm_wday], 
						months[edit_link->repeatEnd.tm_mon], 
						edit_link->repeatEnd.tm_mday,
						edit_link->repeatEnd.tm_year + 1900);
					gtk_entry_set_text(GTK_ENTRY(repeat_entry_text), buf);
				}
			}
			gtk_widget_set_sensitive(GTK_WIDGET(repeat_forever_hbox), True);
			break;
	}
}


/*========================================================================
 *	Name:			RepeatWeekdaySetup()
 *	Prototype:	RepeatWeekdaySetup()
 *					
 *	Description:
 *	When the Repeat Weekday changes, ...
 *========================================================================*/
static void
RepeatWeekdaySetup(
	GtkWidget	*widget,
	int			id
)
{
#ifdef DEBUG
	char	*fname="RepeatWeekdaySetup";
#endif

	DBGEnter();
	if ( gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(repeat_weekday_checkbutton[id])) ) 
	{
		repeat_weekday_set[id] = 1;
	}
	else
	{
		repeat_weekday_set[id] = 0;
	}
}


/*========================================================================
 *	Name:			RepeatUntilSetup
 *	Prototype:	RepeatUntilSetup()
 *					
 *	Description:
 *	One of the Repeat Forever/Until checkbuttons has been toggled.
 *========================================================================*/
static void
RepeatUntilSetup(
	GtkWidget	*widget,
	int			id
)
{
	gtk_signal_handler_block (GTK_OBJECT (repeat_forever_togglebutton), 
		rpf_sig_id);
	gtk_signal_handler_block (GTK_OBJECT (repeat_until_togglebutton), 
		rpu_sig_id);

	if (id == 1)
	{
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(repeat_forever_togglebutton), TRUE);
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(repeat_until_togglebutton), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(repeat_entry_text), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(repeat_cal_button), FALSE);
		gtk_entry_set_text(GTK_ENTRY(repeat_entry_text), "");
	}
	else
	{
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(repeat_until_togglebutton), TRUE);
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(repeat_forever_togglebutton), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(repeat_entry_text), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(repeat_cal_button), TRUE);
	}

	gtk_signal_handler_unblock (GTK_OBJECT (repeat_forever_togglebutton), 
		rpf_sig_id);
	gtk_signal_handler_unblock (GTK_OBJECT (repeat_until_togglebutton), 
		rpu_sig_id);
}


/*========================================================================
 *	Name:			EditDate
 *	Prototype:	EditDate()
 *					
 *	Description:
 *	Edit the date the repeating appointment will end on.
 *========================================================================*/
static void
EditDate()
{
	int		yearnum, monthnum, daynum;
	GDate		*todaydate;

	gtk_calendar_get_date(GTK_CALENDAR(entry_calendar),
				&(yearnum),
				&(monthnum),
				&(daynum)
				);
	todaydate = g_date_new_dmy( daynum, monthnum+1, yearnum);
	ApptCalendar.monthnum = monthnum;
	ApptCalendar.daynum = daynum;
	ApptCalendar.yearnum = yearnum;
	ApptCalendar.wdnum = g_date_weekday(todaydate);
	ApptCalendar.callback = &ApptCalCb;
	ApptCalendar.cbdata = NULL;
	ApptCalendar.nodatecb = &ApptNoDateCb;
	g_date_free(todaydate);

	XPShowCalendar(&ApptCalendar, 1);
}

/*========================================================================
 *	Name:			ApptCalCb
 *	Prototype:	ApptCalCb()
 *					
 *	Description:
 *	Handle a selected date from the calendar.
 *========================================================================*/
static void
ApptCalCb()
{
	char	buf[128];

	/*
	 * Get the calendar date.
	 */
	XPCalendarGetDate(&ApptCalendar);
	sprintf(buf, "%s %s %d, %d",
		weekdays[ApptCalendar.wdnum],
		months[ApptCalendar.monthnum],
		ApptCalendar.daynum,
		ApptCalendar.yearnum);
	gtk_entry_set_text(GTK_ENTRY(repeat_entry_text), buf);
	XPCloseCalendar();
}

/*========================================================================
 *	Name:			ApptNoDateCb
 *	Prototype:	ApptNoDateCb()
 *					
 *	Description:
 *	Handle a no selected date from the calendar.
 *========================================================================*/
static void
ApptNoDateCb()
{
	XPCloseCalendar();
}

/*========================================================================
 *	Name:			CloseNewAppt
 *	Prototype:	CloseNewAppt()
 *					
 *	Description:
 *	Close the "New Appointment" dialog.
 *========================================================================*/
static void
CloseNewAppt()
{
	/*
	 * If the Edit Note window is opened, close it.
	 */
	EditNoteCancel(NULL);

	/*
	 * Close the dialog.
	 */
	if ( newapptdialog != NULL )
	{
		gtk_widget_hide( newapptdialog );
	}
	edit_link = NULL;
}

/*========================================================================
 *	Name:			AcceptAppt
 *	Prototype:	AcceptAppt()
 *					
 *	Description:
 *	Accept the updated entry in the Edit Appointment dialog.
 *========================================================================*/
static void
AcceptAppt()
{
	int	yearnum, monthnum, daynum;
	int	i;
	char	buf[MAXPATHLEN+1];
	char	*value;

	PilotAppt_T		*pentry;
	PilotAppt_T		*current_link;
	GDate				*newdate;
	int				weekday;

	
	/* Grab the data from the dialog. */
	gtk_calendar_get_date(GTK_CALENDAR(entry_calendar),
				&(yearnum),
				&(monthnum),
				&(daynum)
				);
	newdate = g_date_new_dmy(daynum, monthnum+1, yearnum);
	weekday = g_date_weekday(newdate);
	if (weekday == 7 ) 
		weekday = 0;
	g_date_free(newdate);

	if ( edit_link != NULL )
	{
		pentry = edit_link;
		/* Reset the edit link - it gets set externally to this routine. */
		edit_link = NULL;
	}
	else
	{
		/* Allocate a new entry. */
		pentry = (PilotAppt_T *)XPSafeAlloc(sizeof(PilotAppt_T));
		memset(pentry,0,sizeof(PilotAppt_T));

		/* Add it to the link list */
		if ( Pilot_Appt )
		{
			current_link = Pilot_Appt;
			while (current_link->next != NULL)
				current_link = current_link->next;
			current_link->next = pentry;
		}
		else
			Pilot_Appt = pentry;
	}


	/* Update the entry (newly allocated or existing). */
	if ( start_ampm == 0 )
		pentry->begin.tm_hour = start_hour;
	else
		pentry->begin.tm_hour = start_hour+12;
	pentry->begin.tm_min = start_min * 15;
	if ( end_ampm == 0 )
		pentry->end.tm_hour = end_hour;
	else
		pentry->end.tm_hour = end_hour+12;
	pentry->end.tm_min = end_min * 15;

	pentry->begin.tm_year = yearnum - 1900;
	pentry->begin.tm_mon = monthnum;
	pentry->begin.tm_mday = daynum;
	pentry->begin.tm_wday = weekday;

	pentry->end.tm_year = yearnum - 1900;
	pentry->end.tm_mon = monthnum;
	pentry->end.tm_mday = daynum;
	pentry->end.tm_wday = weekday;

	/* There are two repeat types we don't support yet */
	if ( repeat_options_set == 4 )
		pentry->repeatType = repeat_options_set+1;
	else
		pentry->repeatType = repeat_options_set;

	/* Repeat forever or until... */
	if ( repeat_options_set != 0 )
	{
		if ( gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(repeat_forever_togglebutton)) )
		{
			pentry->repeatForever = 1;
			bzero((char *)&pentry->repeatEnd, sizeof(struct tm));
		}
		else
		{
			pentry->repeatForever = 0;
			pentry->repeatEnd.tm_year = ApptCalendar.yearnum - 1900;
			pentry->repeatEnd.tm_mon = ApptCalendar.monthnum;
			pentry->repeatEnd.tm_mday = ApptCalendar.daynum;
			pentry->repeatEnd.tm_wday = ApptCalendar.wdnum-1;
		}
	}
	else
	{
		/* No repeat specified, so clear these fields. */
		pentry->repeatForever = 1;
		bzero((char *)&pentry->repeatEnd, sizeof(struct tm));
	}
	
	for (i=0; i<7; i++)
		pentry->repeatDays[i] = repeat_weekday_set[i];

	pentry->alarm = alarm_state;
	pentry->advance = 
		gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(alarm_advamount_spinbox));
	pentry->advanceUnits = alarm_type;

	if ( pentry->description )
		free(pentry->description);
	value = gtk_editable_get_chars(GTK_EDITABLE(cal_entry_text), 0, -1);
	pentry->description = (char *)malloc(strlen(value) + 1);
	strcpy(pentry->description, value);
	free(value);

	if ( textnote )
	{
		if ( pentry->note ) free(pentry->note);
		pentry->note = gtk_editable_get_chars(GTK_EDITABLE(textnote), 0, -1);
	}

	/* Close the dialog and update all pages of the notebook. */
	CloseNewAppt();
	PopulateCalendar();

	/* Save all changes to file */
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPApptFile);
	XPPilotWriteAppointment(buf);
}

/*========================================================================
 *	Name:			DeleteAppt
 *	Prototype:	DeleteAppt()
 *					
 *	Description:
 *	Delete the current entry from the Edit Appointment dialog.
 *========================================================================*/
static void
DeleteAppt()
{
	PilotAppt_T		*current_link;
	char				buf[MAXPATHLEN+1];

	if ( edit_link != NULL )
	{
		/* Remove the link from the list first. */
		if ( Pilot_Appt != edit_link )
		{
			current_link = Pilot_Appt;
			while (current_link->next != edit_link)
				current_link = current_link->next;
			current_link->next = edit_link->next;
		}
		else
			Pilot_Appt = edit_link->next;

		/* Freeup link */
		if ( edit_link->description ) free(edit_link->description);
		if ( edit_link->note ) free(edit_link->note);
		if ( edit_link->exceptions ) free((char *)edit_link->exceptions);
	}
	CloseNewAppt();

	/* Update the display. */
	PopulateCalendar();

	/* Save all changes to file */
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPApptFile);
	XPPilotWriteAppointment(buf);
}

/*========================================================================
 *	Name:			CloseAppt
 *	Prototype:	CloseAppt()
 *					
 *	Description:
 *	Close the Calendar dialog.
 *========================================================================*/
static void
CloseAppt()
{
	/*
	 * If the Edit Note window is opened, close it.
	 */
	EditNoteCancel(NULL);

	/*
	 * If the New Appointment window is opened, close it.
	 */
	CloseNewAppt();

	/*
	 * Close the dialog.
	 */
	if ( appt_widget != NULL )
	{
		gtk_widget_hide( appt_widget );
	}
}


/*========================================================================
 *	Name:			ApptSelectDay
 *	Prototype:	ApptSelectDay()
 *					
 *	Description:
 *	Update the display based on a new day selection.
 *========================================================================*/
static void
ApptSelectDay(
	GtkWidget	*widget,
	int			type
)
{
	char	buf[128];
	int	yearnum, monthnum, daynum;
	GDate	*weekdate;

	switch(type)
	{
		/* "Today" calendar changed */ 
		case 1:	
			/* Get the day, month, year from the calendar */
			gtk_calendar_get_date(GTK_CALENDAR(calendar),
				&(yearnum),
				&(monthnum),
				&(daynum)
				);

			if ( currentdate ) g_date_free(currentdate);
			currentdate = g_date_new_dmy( daynum, monthnum+1, yearnum);
			weekdate = g_date_new_dmy( daynum, monthnum+1, yearnum);

			sprintf(buf, "%s %s %s %d, %d", 
				XP_S_APPT_SUMMARY, 
				weekdays[g_date_weekday(currentdate)], months[monthnum], daynum, yearnum);
			gtk_label_set_text(GTK_LABEL(todaylabel), buf);

			g_date_subtract_days( weekdate, g_date_weekday(currentdate));
			sprintf(buf, "%s %s %d, %d", 
				XP_S_APPT_WEEK_SUMMARY, 
				months[g_date_month(weekdate)-1], g_date_day(weekdate), 
				g_date_year(weekdate));
			g_date_free(weekdate);
			gtk_label_set_text(GTK_LABEL(weeklabel), buf);

			sprintf(buf, "%s %s %d", 
				XP_S_APPT_MONTH_SUMMARY, months[monthnum], yearnum);
			gtk_label_set_text(GTK_LABEL(monthlabel), buf);

			gtk_signal_handler_block (GTK_OBJECT (calendar_week), 
				calendar_week_sig_id);
			gtk_calendar_select_day(GTK_CALENDAR(calendar_week), daynum);
			gtk_calendar_select_month(GTK_CALENDAR(calendar_week), 
				monthnum, yearnum);
			gtk_signal_handler_unblock (GTK_OBJECT (calendar_week), 
				calendar_week_sig_id);
			break;

		/* "Weekly" calendar changed */ 
		case 2:	
			/* Get the day, month, year from the calendar */
			gtk_calendar_get_date(GTK_CALENDAR(calendar_week),
				&(yearnum),
				&(monthnum),
				&(daynum)
				);

			if ( currentdate ) g_date_free(currentdate);
			currentdate = g_date_new_dmy( daynum, monthnum+1, yearnum);
			weekdate = g_date_new_dmy( daynum, monthnum+1, yearnum);

			sprintf(buf, "%s %s %s %d, %d", 
				XP_S_APPT_SUMMARY, 
				weekdays[g_date_weekday(currentdate)], months[monthnum], 
				daynum, yearnum);
			gtk_label_set_text(GTK_LABEL(todaylabel), buf);

			g_date_subtract_days( weekdate, g_date_weekday(currentdate));
			sprintf(buf, "%s %s %d, %d", 
				XP_S_APPT_WEEK_SUMMARY, 
				months[g_date_month(weekdate)-1], g_date_day(weekdate), 
				g_date_year(weekdate));
			g_date_free(weekdate);
			gtk_label_set_text(GTK_LABEL(weeklabel), buf);

			sprintf(buf, "%s %s %d", XP_S_APPT_MONTH_SUMMARY, 
				months[monthnum], yearnum);
			gtk_label_set_text(GTK_LABEL(monthlabel), buf);

			gtk_signal_handler_block (GTK_OBJECT (calendar), calendar_sig_id);
			gtk_calendar_select_day(GTK_CALENDAR(calendar), daynum);
			gtk_calendar_select_month(GTK_CALENDAR(calendar), monthnum, yearnum);
			gtk_signal_handler_unblock (GTK_OBJECT (calendar), calendar_sig_id);
			break;

		/* "New Entry" calendar changed */ 
		case 3:	
			/* Get the day, month, year from the calendar */
			ResetEditDialog();
			break;
	}

	PopulateCalendar();
}

/*========================================================================
 *	Name:			PageChange
 *	Prototype:	PageChange()
 *					
 *	Description:
 *	Update the display with current information.
 *========================================================================*/
static void
PageChange(
	GtkNotebook *notebook,
	GtkNotebookPage *page,
	gint page_num,
	gpointer user_data
)
{
	pageid = page_num;
	PopulateCalendar();
}

/*========================================================================
 *	Name:			ChooseWeekDay
 *	Prototype:	ChooseWeekDay()
 *					
 *	Description:
 *	Handle a selection made in the Weekly page.
 *========================================================================*/
static void
ChooseWeekDay(
	GtkWidget	*list_widget,
	gpointer		function_data
)
{
	PilotAppt_T	*current_link;
	GList			*list;
	GtkWidget	*list_item;

	list = (GTK_LIST(list_widget))->selection;
	if ( list == NULL )
		return;
	list_item = GTK_WIDGET(list->data);
	current_link = (PilotAppt_T *)gtk_object_get_user_data(GTK_OBJECT(list_item));
	
	/* Get the date selected. */
	g_date_free(currentdate);
	currentdate = g_date_new_dmy( 
							current_link->begin.tm_mday,
							current_link->begin.tm_mon+1,
							current_link->begin.tm_year+1900
						);
	gtk_signal_handler_block (GTK_OBJECT (calendar), calendar_sig_id);
	gtk_signal_handler_block (GTK_OBJECT (calendar_week), calendar_week_sig_id);
	gtk_calendar_select_day(GTK_CALENDAR(calendar), current_link->begin.tm_mday);
	gtk_calendar_select_day(GTK_CALENDAR(calendar_week), current_link->begin.tm_mday);
	gtk_signal_handler_unblock (GTK_OBJECT (calendar), calendar_sig_id);
	gtk_signal_handler_unblock (GTK_OBJECT (calendar_week), calendar_week_sig_id);

	/* Change to the first page. */
	gtk_notebook_set_page(GTK_NOTEBOOK(monthlynotebook), 0);

	/* Update the display. */
	PopulateCalendar();
}

/*========================================================================
 *	Name:			ChooseToday
 *	Prototype:	ChooseToday()
 *					
 *	Description:
 *	Handle a selection made in the Today page.
 *========================================================================*/
static void
ChooseToday(
   GtkWidget      *widget,
   gint           row,
   gint           column,
   GdkEventButton *event,
   gpointer       data
)
{
#ifdef DEBUG
	char	fname[]="ChooseToday";
#endif

	PilotAppt_T	*current_link;
	int			yearnum;
	int			monthnum;
	int			daynum;
	char			buf[128];
	int			i;

	DBGEnter();

	if ( newapptdialog == NULL )
		CreateNewApptDialog();

	current_link = (PilotAppt_T *)gtk_clist_get_row_data(GTK_CLIST(widget), row);
	if ( current_link == NULL ) return;
	gtk_clist_unselect_all(GTK_CLIST(widget));

	/* Fill the NewEntry dialog with the values from the current_link. */
	
	if ( current_link->description )
		gtk_entry_set_text(GTK_ENTRY(cal_entry_text), current_link->description);

	gtk_calendar_get_date(GTK_CALENDAR(calendar_week),
				&(yearnum),
				&(monthnum),
				&(daynum)
				);
	gtk_calendar_select_day(GTK_CALENDAR(entry_calendar), daynum);
	gtk_calendar_select_month(GTK_CALENDAR(entry_calendar), monthnum, yearnum);

	/* There is one montly repeat type we don't yet support. */
	if (current_link->repeatType >= XP_PA_REPEATNOTSUPPORTED)
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(
				repeat_options_radiobutton[current_link->repeatType-1]), True);
	else
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(
				repeat_options_radiobutton[current_link->repeatType]), True);

	for (i=0; i<7; i++)
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(repeat_weekday_checkbutton[i]),
				current_link->repeatDays[i]);

	if (current_link->repeatType != XP_PA_REPEATNONE)
	{
		if (current_link->repeatForever)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(repeat_forever_togglebutton), True);
			gtk_entry_set_text(GTK_ENTRY(repeat_entry_text), "");
		}
		else
		{
			gtk_toggle_button_set_active(
					GTK_TOGGLE_BUTTON(repeat_until_togglebutton), True);

			/* Better make sure we don't have an invalide Until date! */
			if ( current_link->repeatEnd.tm_mday == 0 )
			{
				current_link->repeatEnd.tm_mon = g_date_month(currentdate);
				current_link->repeatEnd.tm_mday = g_date_day(currentdate);
				current_link->repeatEnd.tm_year = g_date_year(currentdate)-1900;
				current_link->repeatEnd.tm_wday = g_date_weekday(currentdate);

				sprintf(buf, "%s %s %d, %d", 
					weekdays[current_link->repeatEnd.tm_wday], 
					months[current_link->repeatEnd.tm_mon], 
					current_link->repeatEnd.tm_mday,
					current_link->repeatEnd.tm_year + 1900);
				gtk_entry_set_text(GTK_ENTRY(repeat_entry_text), buf);
			}
			else
			{
				sprintf(buf, "%s %s %d, %d", 
					weekdays[current_link->repeatEnd.tm_wday], 
					months[current_link->repeatEnd.tm_mon], 
					current_link->repeatEnd.tm_mday,
					current_link->repeatEnd.tm_year + 1900);
				gtk_entry_set_text(GTK_ENTRY(repeat_entry_text), buf);
			}
		}
	}

	if ( current_link->begin.tm_hour < 12 )
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (start_hour_optionmenu), 
			current_link->begin.tm_hour);
		gtk_option_menu_set_history (GTK_OPTION_MENU (start_ampm_optionmenu), 0);
		start_hour = current_link->begin.tm_hour;
		start_ampm = 0;
	}
	else
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (start_hour_optionmenu), 
			current_link->begin.tm_hour - 12);
		gtk_option_menu_set_history (GTK_OPTION_MENU (start_ampm_optionmenu), 1);
		start_hour = current_link->begin.tm_hour - 12;
		start_ampm = 1;
	}

	if ( current_link->begin.tm_min < 15 )
	{
		gtk_option_menu_set_history (
				GTK_OPTION_MENU (start_minute_optionmenu), 0);
		start_min = 0;
	}
	else
	if (( current_link->begin.tm_min >= 15 ) && 
		 ( current_link->begin.tm_min < 30 ))
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (start_minute_optionmenu), 1);
		start_min = 1;
	}
	else
	if (( current_link->begin.tm_min >= 30 ) && ( current_link->begin.tm_min < 45 ))
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (start_minute_optionmenu), 2);
		start_min = 2;
	}
	else
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (start_minute_optionmenu), 3);
		start_min = 3;
	}

	if ( current_link->end.tm_hour < 12 )
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_hour_optionmenu), 
			current_link->end.tm_hour);
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_ampm_optionmenu), 0);
		end_hour = current_link->end.tm_hour;
		end_ampm = 0;
	}
	else
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_hour_optionmenu), 
			current_link->end.tm_hour - 12);
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_ampm_optionmenu), 1);
		end_hour = current_link->end.tm_hour - 12;
		end_ampm = 1;
	}

	if ( current_link->end.tm_min < 15 )
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_minute_optionmenu), 0);
		end_min = 0;
	}
	else
	if (( current_link->end.tm_min >= 15 ) && ( current_link->end.tm_min < 30 ))
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_minute_optionmenu), 1);
		end_min = 1;
	}
	else
	if (( current_link->end.tm_min >= 30 ) && ( current_link->end.tm_min < 45 ))
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_minute_optionmenu), 2);
		end_min = 2;
	}
	else
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (end_minute_optionmenu), 3);
		end_min = 3;
	}

	DBGPrintf(DBG_INFO, ("alarm/adv/opt: %d / %d / %d\n", 
			current_link->alarm,
			current_link->advance,
			current_link->advanceUnits));
	if ( current_link->alarm )
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alarm_on_radiobutton), 
			True);
		gtk_adjustment_set_value(GTK_ADJUSTMENT(alarm_adv_adj), 
			current_link->advance);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(alarm_advamount_spinbox), 
			(gfloat)current_link->advance);
		gtk_option_menu_set_history (GTK_OPTION_MENU (alarm_type_optionmenu), 
			current_link->advanceUnits);
	}
	alarm_state = current_link->alarm;
	alarm_type  = current_link->advanceUnits;

	sprintf(buf, "%s %s %s %d, %d", 
				XP_S_APPT_SUMMARY, 
				weekdays[g_date_weekday(currentdate)], 
				months[monthnum], daynum, yearnum);
	gtk_label_set_text(GTK_LABEL(entry_label), buf);

	/* Display the NewEntry dialog. */
	edit_link = current_link;

	gtk_widget_show(newappt_delete_button);
	gtk_widget_show(newapptdialog);
}

/*========================================================================
 *	Name:			ChooseMonthDay
 *	Prototype:	ChooseMonthDay()
 *					
 *	Description:
 *	Handle a selection made in the Monthly page.
 *========================================================================*/
static void
ChooseMonthDay(
   GtkWidget      *widget,
   gint           row,
   gint           column,
   GdkEventButton *event,
   gpointer       data
)
{
	PilotAppt_T		*current_link;
	MonthlyCell_T	*linkptr;

	linkptr = (MonthlyCell_T *)gtk_clist_get_row_data(GTK_CLIST(widget), row);
	current_link = (PilotAppt_T *)linkptr->linkptr;
	if ( current_link == NULL ) return;

	g_date_free(currentdate);
	currentdate = g_date_new_dmy( 
							linkptr->date[0],
							linkptr->date[1],
							current_link->begin.tm_year+1900
						);
	gtk_signal_handler_block (GTK_OBJECT (calendar), calendar_sig_id);
	gtk_signal_handler_block (GTK_OBJECT (calendar_week), calendar_week_sig_id);
	gtk_calendar_select_month(GTK_CALENDAR(calendar), 
		linkptr->date[1], current_link->begin.tm_year+1900);
	gtk_calendar_select_day(GTK_CALENDAR(calendar), linkptr->date[0]);
	gtk_calendar_select_month(GTK_CALENDAR(calendar_week), 
		linkptr->date[1], current_link->begin.tm_year+1900);
	gtk_calendar_select_day(GTK_CALENDAR(calendar_week), linkptr->date[0]);
	gtk_signal_handler_unblock (GTK_OBJECT (calendar), calendar_sig_id);
	gtk_signal_handler_unblock (GTK_OBJECT (calendar_week), 
		calendar_week_sig_id);

	/* Update the titles, etc. */
	ApptSelectDay(NULL, 1);

	/* Change to the first page. */
	gtk_notebook_set_page(GTK_NOTEBOOK(monthlynotebook), 0);

	/* Update the display. */
	PopulateCalendar();
}

/*========================================================================
 *	Name:			PopulateCalendar
 *	Prototype:	PopulateCalendar()
 *					
 *	Description:
 *	Update the display with current information.
 *========================================================================*/
static void
PopulateCalendar()
{
#ifdef DEBUG
	char	fname[]="PopulateCalendar";
#endif

	GDate			*offsetdate;
	int			lastday;
	PilotAppt_T	*current_link;
	char			row_data[3][1024];
	char			*rows[3];
	int			row;
	int			dayid;
	int			i;
	int			cell;
	int			index;
	GdkPixmap 	*xbm_ptr;
	GdkBitmap 	*mask_ptr;
	GtkWidget	*list_item;

	MonthlyCell_T	*linkptr;

	DBGEnter();

	/* Its possible to get here early - so just skip it if that happens. */
	if ( currentdate == NULL ) 
		return;

	switch (pageid)
	{
		/* Daily page */
		case 0:
			/* Clear the current clist */
			gtk_clist_clear(GTK_CLIST(todaylist));
			gtk_clist_freeze(GTK_CLIST(todaylist));
			current_link = Pilot_Appt;
			while (current_link != NULL) 
			{
				if ( CheckToday(current_link, currentdate) == XP_E_SUCCESS )
				{
					/* Add a clist row for this entry */
					sprintf(row_data[0], "%02d:%02d", 
						current_link->begin.tm_hour,
						current_link->begin.tm_min);
					sprintf(row_data[1], "%02d:%02d", 
						current_link->end.tm_hour,
						current_link->end.tm_min);
					sprintf(row_data[2], "%s", current_link->description);
					rows[0] = row_data[0];
					rows[1] = row_data[1];
					rows[2] = row_data[2];

					row = gtk_clist_append(GTK_CLIST(todaylist), rows);
					gtk_clist_set_row_data(GTK_CLIST(todaylist), 
						row, (gpointer)current_link);
				}
				current_link = current_link->next;
			}

			/* Now sort the displayed list. */
			gtk_clist_sort(GTK_CLIST(todaylist));
			gtk_clist_thaw(GTK_CLIST(todaylist));
			break;

		/* Weekly page */
		case 1:
			/* Clear the current clist */
			for(i=0;i<7;i++) gtk_list_clear_items(GTK_LIST(weeklists[i]), 0, -1);

			/* Get a copy of the current date and substract the offset. */
			offsetdate = g_date_new_dmy(
									g_date_day(currentdate),
									g_date_month(currentdate),
									g_date_year(currentdate));

			/* Get the offset from the begining of the week, which is Monday */
			dayid = g_date_weekday(currentdate);
			if ( dayid != 7 )
			{
				/* 1-6 are M-Sat, Sun is 7, so don't substract anything */
				dayid = -1 * dayid;
				g_date_add_days(offsetdate, dayid);
			}

			current_link = Pilot_Appt;
			while (current_link != NULL) 
			{
				for(i=0; i<7; i++)
				{
					if ( CheckToday(current_link, offsetdate) == XP_E_SUCCESS )
					{
						if (current_link->description)
							list_item = 
								gtk_list_item_new_with_label 
									(current_link->description);
						else
							list_item = 
								gtk_list_item_new_with_label ("No Description");
						gtk_container_add (GTK_CONTAINER (weeklists[i]), list_item);
						gtk_widget_show (list_item);
						gtk_object_set_user_data (GTK_OBJECT (list_item), 
							(gpointer) current_link);
					}
					g_date_add_days(offsetdate, 1);
				}

				/* Get next link. */
				current_link = current_link->next;
				g_date_add_days(offsetdate, -7);
			}
			g_date_free(offsetdate);
			break;

		/* Monthly page */
		case 2:
			SetUpMonthlyPage(g_date_month(currentdate)-1, 
				g_date_year(currentdate));

			/* Get a copy of the current month/year, starting on the 1st. */
			offsetdate = g_date_new_dmy(
									1,
									g_date_month(currentdate),
									g_date_year(currentdate));

			/*
			 * Get the offset from the begining of the week, which is Monday.
			 * This is the offsets into the Month clists array.
			 */
			index = g_date_weekday(offsetdate);
			if ( index == 7 )
				index = 0;

			/*
			 * Get the last day of the month.
			 */
			lastday = g_date_days_in_month(
							g_date_month(offsetdate), 
							g_date_year(offsetdate));

			DBGPrintf(DBG_INFO, ("Monthly Start date: %d / %d / %d\n", 
					g_date_year(offsetdate),
					g_date_month(offsetdate),
					g_date_day(offsetdate)));

			current_link = Pilot_Appt;
			while (current_link != NULL) 
			{
				for(i=0+index; i<0+index+lastday; i++)
				{
					if ( CheckToday(current_link, offsetdate) == XP_E_SUCCESS )
					{
						/* Add this link to the moth */
						if ( current_link->begin.tm_hour < 12 )
						{
							xbm_ptr = cal_morning_xbm;
							mask_ptr = cal_morning_mask;
							cell = 1;
						}
						else
						if (( current_link->begin.tm_hour >= 12 ) &&
						    ( current_link->begin.tm_hour < 18 ))
						{
							xbm_ptr = cal_noon_xbm;
							mask_ptr = cal_noon_mask;
							cell = 2;
						}
						else
						{
							xbm_ptr = cal_night_xbm;
							mask_ptr = cal_night_mask;
							cell = 3;
						}
						gtk_clist_set_pixmap(GTK_CLIST(monthlists[i]), cell, 0, 
							xbm_ptr, mask_ptr);
						gtk_clist_set_selectable (GTK_CLIST (monthlists[i]), 
							cell, TRUE);
						linkptr = 
							(MonthlyCell_T *)gtk_clist_get_row_data(
								GTK_CLIST(monthlists[i]), cell);
						linkptr->linkptr = current_link;
						linkptr->date[0] = g_date_day(offsetdate);
						linkptr->date[1] = g_date_month(offsetdate)-1;
					}

					/* Move to the next date */
					g_date_add_days(offsetdate, 1);
				}
				current_link = current_link->next;
				g_date_add_days(offsetdate, (-1 * lastday));
			}
			break;
	}
}


/*========================================================================
 *	Name:			CheckToday
 *					
 *	Description:
 *	This function checks if an appointment should be displayed in one of
 * the three pages of the notebook.
 *
 * Returns:
 * XP_E_SUCCESS if appointment should be displayed.
 * XP_E_FAILURE if appointment should not be displayed.
 *========================================================================*/
static int
CheckToday(
	PilotAppt_T *current_link,
	GDate			*initial_date
)
{
#ifdef DEBUG
	char	fname[]="CheckToday";
#endif

	GDate			*startdate = NULL;
	GDate			*untildate = NULL;
	int			state = XP_E_FAILURE;
	char			buf[128];

	/* Assume it won't get displayed. */
	state = XP_E_FAILURE;

	/* Get the gdate for the entry */
	DBGPrintf(DBG_INFO, ("Startdate (%d/%d): %d / %d / %d\n", 
							current_link->repeatType, XP_PA_REPEATNONE,
							current_link->begin.tm_mday,
							current_link->begin.tm_mon+1,
							current_link->begin.tm_year+1900));
	startdate = g_date_new_dmy(
							current_link->begin.tm_mday,
							current_link->begin.tm_mon+1,
							current_link->begin.tm_year+1900);
	g_date_strftime(buf, 128, "%a %b %d %T %Y", initial_date);
	DBGPrintf(DBG_INFO, ("Compare with: %s\n", buf));

	/* if no repeat and start() = today, include it */
	if ( current_link->repeatType == XP_PA_REPEATNONE )
	{
		DBGPrintf(DBG_INFO, ("XP_PA_REPEATNONE\n"));
		if ( g_date_compare(startdate, initial_date) == 0 )
			state = XP_E_SUCCESS;
		goto skiptoday;
	}

	/*
	 * if repeat = daily: 
	 *		if (forever or until >=today) and start() <= today, include it
	 */
	if ( current_link->repeatType == XP_PA_REPEATDAILY )
	{
		DBGPrintf(DBG_INFO, ("XP_PA_REPEATDAILY\n"));
		if ( g_date_compare(startdate, initial_date) <= 0)
		{
			if ( current_link->repeatForever ) 
				state = XP_E_SUCCESS;
			else
			{
				/* Get the date of the "repeatUntil" */
				untildate = g_date_new_dmy(
						current_link->repeatEnd.tm_mday,
						current_link->repeatEnd.tm_mon+1,
						current_link->repeatEnd.tm_year+1900);

				if ( g_date_compare(untildate, initial_date) >= 0 )
					state = XP_E_SUCCESS;
			}
		}
		goto skiptoday;
	}

	/*
	 * if repeat is weekly:
	 *		if (forever or until >=today) and start() <= today and
	 *			weekdays() = today, include it
	 */
	if ( current_link->repeatType == XP_PA_REPEATWEEKLY )
	{
		DBGPrintf(DBG_INFO, ("XP_PA_REPEATWEEKLY\n"));
		if ( g_date_compare(startdate, initial_date) <= 0)
		{
			if ( current_link->repeatForever )
			{
				if (( current_link->repeatDays[g_date_weekday(initial_date)] ) ||
					 ( current_link->begin.tm_mday == g_date_day(initial_date) ))
					state = XP_E_SUCCESS;
			}
			else
			{
				/* Get the date of the "repeatUntil" */
				untildate = g_date_new_dmy(
						current_link->repeatEnd.tm_mday,
						current_link->repeatEnd.tm_mon+1,
						current_link->repeatEnd.tm_year+1900);

				g_date_strftime(buf, 128, "%a %b %d %T %Y", untildate);
				DBGPrintf(DBG_INFO, ("*** Until Date: %s\n", buf));
				g_date_strftime(buf, 128, "%a %b %d %T %Y", startdate);
				DBGPrintf(DBG_INFO, ("*** Start Date: %s\n", buf));
				DBGPrintf(DBG_INFO, ("*** current link m/d: %d/%d\n",
					current_link->begin.tm_mon,
					current_link->begin.tm_mday
					));
				DBGPrintf(DBG_INFO, ("*** initial date m/d: %d/%d\n",
					g_date_month(initial_date),
					g_date_day(initial_date)
					));

				if ( g_date_compare(untildate, initial_date) >= 0 )
				{
					DBGPrintf(DBG_INFO, ("*** until date is okay.\n"));
					if ( (current_link->repeatDays[g_date_weekday(initial_date)]) ||
					  	  ((current_link->begin.tm_mday == g_date_day(initial_date))&&
					  	   (current_link->begin.tm_mon+1 == g_date_month(initial_date))))
					{
						DBGPrintf(DBG_INFO, ("*** returning success.\n"));
						state = XP_E_SUCCESS;
					}
				}
			}
		}
		goto skiptoday;
	}

	/*
	 *	if repeat is monthly:
	 *		if (forever or until >= today) and start() <= today and
	 *			date = today, include it
	 */
	if ( current_link->repeatType == XP_PA_REPEATMONTHLY )
	{
		DBGPrintf(DBG_INFO, ("XP_PA_REPEATMONTHLY\n"));
		if ( g_date_compare(startdate, initial_date) <= 0)
		{
			if ( current_link->repeatForever )
			{
				if ( current_link->begin.tm_mday == g_date_day(initial_date))
					state = XP_E_SUCCESS;
			}
			else
			{
				/* Get the date of the "repeatUntil" */
				untildate = g_date_new_dmy(
						current_link->repeatEnd.tm_mday,
						current_link->repeatEnd.tm_mon+1,
						current_link->repeatEnd.tm_year+1900);

				if ( g_date_compare(untildate, initial_date) >= 0 )
					if ( current_link->begin.tm_mday == g_date_day(initial_date))
						state = XP_E_SUCCESS;
			}
		}
		goto skiptoday;
	}

	/*
	 *	if repeat is yearly:
	 *		if (forever or until >=today) and start() <= today and 
	 *			month/day = today, include it
	 */
	if ( current_link->repeatType == XP_PA_REPEATYEARLY )
	{
		DBGPrintf(DBG_INFO, ("XP_PA_REPEATYEARLY\n"));
		if ( g_date_compare(startdate, initial_date) <= 0)
		{
			if ( current_link->repeatForever )
			{
				if ( ( current_link->begin.tm_mon == g_date_month(initial_date)) &&
					  ( current_link->begin.tm_mday == g_date_day(initial_date)) )
					state = XP_E_SUCCESS;
			}
			else
			{
				/* Get the date of the "repeatUntil" */
				untildate = g_date_new_dmy(
						current_link->repeatEnd.tm_mday,
						current_link->repeatEnd.tm_mon+1,
						current_link->repeatEnd.tm_year+1900);

				if ( g_date_compare(untildate, initial_date) >= 0 )
					if ((current_link->begin.tm_mon == g_date_month(initial_date)) &&
					    (current_link->begin.tm_mday == g_date_day(initial_date)))
						state = XP_E_SUCCESS;
			}
		}
	}

skiptoday:
	/* Cleanup */
	g_date_free(startdate);
	if ( untildate ) g_date_free(untildate);

	/* Return */
	return(state);
}

/*========================================================================
 *	Name:			EditNote
 *	Prototype:	void EditNote()
 *					
 *	Description:
 * Edit the note field.
 *
 *	Notes:
 *========================================================================*/
static void
EditNote(
	GtkWidget	*widget
)
{
	GShell_T				gshell;
	GtkWidget			*hbox, *vbox;
	GtkWidget			*cancel_button, *convert_button;
	GtkWidget			*import_button;
	GtkWidget			*erase_button;
	GtkWidget			*save_button;
	GtkWidget			*text_scrollbar;

	/*
	 * Allow only one note opened for editing at a time.
	 */
	if (editnotewidget != NULL )
	{
		if ( edit_link )
		{
			EraseNote();
			gtk_text_freeze(GTK_TEXT(textnote));
			gtk_text_insert (
				GTK_TEXT (textnote), 	
				NULL, 						
				&textnote->style->black,
				NULL,							
				edit_link->note, 			
				-1);							
			gtk_text_thaw (GTK_TEXT (textnote));
		}
		gtk_widget_show(editnotewidget);
		return;
	}

	/*
	 * Create a generic dialog.  It needs buttons for Save and Cancel.
	 */
	editnotewidget = XPCreateGenericShell(EditNoteDialog,
			OK_BUTTON | CLOSE_BUTTON | SEARCH_BUTTON | 
			OVERWRITE_BUTTON | APPEND_BUTTON, 
			&gshell, FALSE );

	vbox = gshell.vbox;
	save_button = gshell.accept;
	cancel_button = gshell.cancel;
	convert_button = gshell.overwrite;
	import_button = gshell.search;
	erase_button = gshell.append;

	/* Change the label of the Accept Button */
	gtk_label_set_text(GTK_LABEL(GTK_BIN(save_button)->child),XP_S_SAVE);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(import_button)->child),XP_S_IMPORT);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(convert_button)->child),XP_S_EXPORT);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(erase_button)->child),XP_S_ERASE);

	/*
	 * Insert a text widget in the generic dialog and stuff it with the text
	 * from the note.
	 */
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_container_add (GTK_CONTAINER (vbox), hbox);
	gtk_widget_show(hbox);

	textnote = gtk_text_new (NULL, NULL);
	gtk_widget_set_usize(textnote, XP_TODONOTE_W, XP_TODONOTE_H);
	gtk_text_set_editable (GTK_TEXT (textnote), TRUE);
	gtk_text_set_word_wrap(GTK_TEXT(textnote), TRUE);
	gtk_box_pack_start (GTK_BOX (hbox), textnote, FALSE, FALSE, 4);
	gtk_widget_realize(textnote);
	if ( edit_link )
	{
		gtk_text_freeze(GTK_TEXT(textnote));
		gtk_text_insert (
			GTK_TEXT (textnote), 	/* widget */
			NULL, 						/* font */
			&textnote->style->black,/* fg color */
			NULL,							/* bg color */
			edit_link->note, 			/* text to insert */
			-1);							/* -1: use strlen(); >0: # chars to insert */
		gtk_text_thaw (GTK_TEXT (textnote));
	}
	gtk_widget_show(textnote);

	text_scrollbar = gtk_vscrollbar_new(GTK_TEXT(textnote)->vadj);
	gtk_widget_show(text_scrollbar);
	gtk_box_pack_start(GTK_BOX(hbox), text_scrollbar, FALSE, FALSE, 0);
	gtk_widget_set_usize(text_scrollbar, 11, -1);

	/*
	 * Add callbacks for the Save and Cancel buttons.
	 */
	gtk_signal_connect (GTK_OBJECT (save_button), "clicked",
			GTK_SIGNAL_FUNC (EditNoteSave), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			GTK_SIGNAL_FUNC (EditNoteCancel), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (convert_button), "clicked",
			GTK_SIGNAL_FUNC (ConvertToXNote), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (import_button), "clicked",
			GTK_SIGNAL_FUNC (ImportXNotes), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (erase_button), "clicked",
			GTK_SIGNAL_FUNC (EraseNote), (gpointer)0);

	/*
	 * Pop up the window.
	 */
	gtk_widget_realize(editnotewidget);
	XPCenterWindow(editnotewidget, NULL, editnotewidget);
	gtk_widget_show( editnotewidget );

}


/*========================================================================
 *	Name:			EditNoteCancel
 *	Prototype:	void EditNoteCancel()
 *					
 *	Description:
 * Close the Edit Note dialog.  Don't save any changes.
 *
 *	Notes:
 *========================================================================*/
static void
EditNoteCancel(
	GtkWidget	*widget
)
{
	int		len;

	len = gtk_text_get_length(GTK_TEXT(textnote));
	gtk_text_set_point(GTK_TEXT(textnote), 0);
	gtk_text_forward_delete(GTK_TEXT(textnote), len);

	if ( edit_link )
	{
		gtk_text_freeze(GTK_TEXT(textnote));
		gtk_text_insert (
			GTK_TEXT (textnote), 	/* widget */
			NULL, 						/* font */
			&textnote->style->black,/* fg color */
			NULL,							/* bg color */
			edit_link->note, 			/* text to insert */
			-1);							/* -1: use strlen(); >0: # chars to insert */
		gtk_text_thaw (GTK_TEXT (textnote));
	}

	/*
	 * Close the dialog.
	 */
	if ( editnotewidget != NULL )
		gtk_widget_hide( editnotewidget );
}


/*========================================================================
 *	Name:			EditNoteSave
 *	Prototype:	void EditNoteSave()
 *					
 *	Description:
 * Save the note contents to the current appointment.
 *
 *	Notes:
 *========================================================================*/
static void
EditNoteSave(
	GtkWidget	*widget
)
{
	EditNoteCancel(NULL);
}


/*========================================================================
 *	Name:			ConvertToXNote
 *	Prototype:	void ConvertToXNote()
 *					
 *	Description:
 * 
 *
 *	Notes:
 *========================================================================*/
static void
ConvertToXNote(
	GtkWidget	*widget
)
{
	StickyNote		*pn;
	char				*note;

	/*
	 * Generate a new note.
	 */
	XPCreateNewNote(StickyNote_2x3, &pn);

	/* Grab the text from the text field */
	note = gtk_editable_get_chars(GTK_EDITABLE(textnote), 0, -1);

	/*
	 * Clean up any text that might have been initialized.
	 */
	if ( pn->pn_text != NULL )
	{
		free(pn->pn_text);
		pn->pn_text = NULL;
	}
	pn->pn_text = (char *)XPSafeAlloc(strlen(note) + 1);
	sprintf(pn->pn_text, note);

	gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
	gtk_text_insert (
		GTK_TEXT (pn->pn_textwidget),
		NULL, 										/* font */
		&pn->pn_textwidget->style->black, 	/* fg color */
		NULL,											/* bg color */
		note, 										/* text to insert */
		-1);							
	gtk_text_thaw (GTK_TEXT (pn->pn_textwidget));
}

/*========================================================================
 *	Name:			EraseNote
 *	Prototype:	void EraseNote()
 *					
 *	Description:
 * Clear the contents of the note.  
 *
 *	Notes:
 * This is not saved until the user clicks on "Save".
 *========================================================================*/
static void
EraseNote(
)
{
	int	len;

	if (textnote)
	{
		len = gtk_text_get_length(GTK_TEXT(textnote));
		gtk_text_set_point(GTK_TEXT(textnote), 0);
		gtk_text_forward_delete(GTK_TEXT(textnote), len);
	}
}

/*========================================================================
 *	Name:			ImportXNotes()
 *	Prototype:	void ImportXNotes()
 *					
 *	Description:
 * Call the dialog which allows selection of an xnote to import as an
 * appointment note.
 *
 *	Notes:
 *========================================================================*/
static void
ImportXNotes(
)
{
	XPGenericNoteList(StickyNoteList, FALSE, 
		GTK_SELECTION_SINGLE, &ImportXNotesCB, NULL);
}
static void
ImportXNotesCB(
	GtkWidget	*list_widget,
	gpointer		*find_list
)
{
	GList				*list;
	GtkWidget		*list_item;
	StickyNote		*pn;
	char				*note;

	/*
	 * Get the selection list.  There should only be one since we only
	 * allow a single selection.  Then grab the Widget pointer and use
	 * it to get the user data.  Thats the pointer to the note we want.
	 */
	list = (GTK_LIST(list_widget))->selection;
	if ( list == NULL )
		return;
	list_item = GTK_WIDGET(list->data);
	pn = (StickyNote *) gtk_object_get_user_data(GTK_OBJECT(list_item));

	note = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	gtk_text_insert (
		GTK_TEXT (textnote),
		NULL, 										/* font */
		&pn->pn_textwidget->style->black, 	/* fg color */
		NULL,											/* bg color */
		note, 										/* text to insert */
		-1);							
	gtk_text_thaw (GTK_TEXT (textnote));
	free(note);
}

/*========================================================================
 *	Name:			XPApptAlarmCheck
 *					
 *	Description:
 * Check the appointments for any alarms that need to go off.
 *========================================================================*/
int
XPApptAlarmCheck()
{
#ifdef DEBUG
	char	fname[]="XPApptAlarmCheck";
#endif

	PilotAppt_T		*current_link;
	GDate				*todaydate;
	GDate				*cmpdate;
	time_t			now;
	time_t			eventtime1;
	time_t			eventtime2;
	struct tm		*tm_tp;
	struct tm		tm_copy;
	char				buf[128];
	char				begin[128];
	gboolean			soundalarm;

	DBGEnter();

	time(&now);
	tm_tp = localtime(&now);

	strftime(buf, 128, "%a %b %d, %Y @ %l:%M %p", tm_tp);
	DBGPrintf(DBG_INFO, ("%s\n", buf));

	todaydate = 
		g_date_new_dmy( tm_tp->tm_mday, tm_tp->tm_mon+1, tm_tp->tm_year+1900);
	current_link = Pilot_Appt;
	while(current_link != NULL)
	{
		/* Did the user ask for an alarm on this appointment? */
		if ( current_link->alarm )
		{
			soundalarm = False;

			/* Get the time once again, to be as accurate as possible. */
			time(&now);
			tm_tp = localtime(&now);

			/*
			 * First, check if the alarm should go off even without advance
			 * notice. 
			 */
			if ( ( tm_tp->tm_year == current_link->begin.tm_year ) &&
				  ( tm_tp->tm_mon == current_link->begin.tm_mon ) &&
				  ( tm_tp->tm_mday == current_link->begin.tm_mday ) &&
				  ( tm_tp->tm_hour == current_link->begin.tm_hour ) &&
				  ( tm_tp->tm_min == current_link->begin.tm_min ) )
			{
				current_link->alarm = 0;
				soundalarm = True;
			}

			/*
			 * If we aren't yet going to sound the alarm, set up for 
			 * further testing.
			 */
			if ( soundalarm == False )
			{
				bcopy((char *)&current_link->begin, (char *)&tm_copy,
					sizeof(struct tm));
				tm_copy.tm_sec = 0;
				tm_tp->tm_sec = 0;
			}

			/* Compute the alarm advance time based on the advanceUnits. */

			/* Minutes */
			if ( (current_link->advance) && (current_link->advanceUnits== 0) &&
				  ( soundalarm == False ) )
			{
				tm_tp->tm_min += current_link->advance;

				DBGPrintf(DBG_INFO, ("Min adv: %s:\n", asctime(tm_tp)));
				DBGPrintf(DBG_INFO, ("Min req: %s:\n", asctime(&tm_copy)));

				/* mktime() will normalize the date/hours/etc. */
				eventtime1 = mktime(&tm_copy);
				eventtime2 = mktime(tm_tp);

				DBGPrintf(DBG_INFO, ("Min eventtime1: %d:\n",eventtime1 ));
				DBGPrintf(DBG_INFO, ("Min eventtime2: %d:\n",eventtime2 ));

				if (eventtime1 == eventtime2)
				{
					soundalarm = True;
					current_link->advance = 0;
				}
			}

			/* Hours */
			if ( (current_link->advance) && (current_link->advanceUnits== 1) &&
				  ( soundalarm == False ) )
			{
				tm_tp->tm_hour += current_link->advance;

				DBGPrintf(DBG_INFO, ("Hour adv: %s:\n", asctime(tm_tp)));
				DBGPrintf(DBG_INFO, ("Hour req: %s:\n", asctime(&tm_copy)));

				/* mktime() will normalize the date/hours/etc. */
				eventtime1 = mktime(&tm_copy);
				eventtime2 = mktime(tm_tp);

				DBGPrintf(DBG_INFO, ("Hour eventtime1: %d:\n",eventtime1 ));
				DBGPrintf(DBG_INFO, ("Hour eventtime2: %d:\n",eventtime2 ));

				if (eventtime1 == eventtime2)
				{
					soundalarm = True;
					current_link->advance = 0;
				}
			}
	
			/* Advance Days are checked differently. */
			if ( (current_link->advance) && (current_link->advanceUnits == 2) &&
				  ( soundalarm == False ) )
			{
				cmpdate = 
					g_date_new_dmy( 
						tm_tp->tm_mday, tm_tp->tm_mon+1, 
							tm_tp->tm_year+1900);
				g_date_add_days(cmpdate, current_link->advance);

				DBGPrintf(DBG_INFO, ("Day adv: checking if days match\n"));
				if ( CheckToday(current_link, cmpdate) == XP_E_SUCCESS )
				{
					DBGPrintf(DBG_INFO, ("Day adv: days do match\n"));
					bcopy((char *)&current_link->begin, (char *)&tm_copy,
						sizeof(struct tm));

					DBGPrintf(DBG_INFO, ("current h/m: %d/%d\n",
						tm_tp->tm_hour, tm_tp->tm_min));
					DBGPrintf(DBG_INFO, ("event h/m: %d/%d\n",
						tm_copy.tm_hour, tm_copy.tm_min));

					/* Just check the hour and minute for matches */
					if ( (tm_tp->tm_hour == tm_copy.tm_hour) && 
						  (tm_tp->tm_min == tm_copy.tm_min) )
					{
						soundalarm = True;
						current_link->advance = 0;
					}
				}
			}

			/* Check to see if the alarm should go off. */
			if ( soundalarm )
			{
				/* Sound the alarm. */
				strftime(begin, 128, "%a %b %d, %Y @ %l:%M %p", 
					&current_link->begin);
				if ( current_link->description )
					sprintf(buf, "%s\n%s\nSummary: %s", 
						XP_S_APPT_ALARM_EVENT, 
						begin,
						current_link->description);
				else
					sprintf(buf, "%s\n%s",
						XP_S_APPT_ALARM_EVENT, 
						begin);
				XPMsgWindow(
					buf, 
					XP_INFO_TYPE | XP_MSG_NOACCEPT | XP_MSG_NOHELP, NULL,
					NULL, NULL, NULL, NULL, NULL);
			}
		}
	
		/* Get the next appointment */
		current_link = current_link->next;
	}
	free(todaydate);

	/* Do this to keep the timer running. */
	return(TRUE);
}

/*========================================================================
 *	Name:			AKeyPress
 *	Prototype:	void AKeyPress()
 *					
 *	Description:
 * Check for ENTER key in text entry fields.
 *
 *	Notes:
 *========================================================================*/
static void
AKeyPress(
	GtkWidget	*widget,
	GdkEventKey	*event,
	int			index
)
{
	switch (event->keyval)
	{
		case GDK_Return:
			AcceptAppt();
			break;
		default:
			break;
	}
}

/*========================================================================
 *	Name:			XPApptInitDayAlarm, DayCheckOne
 *	Prototype:	void XPApptInitDayAlarm()
 *					
 *	Description:
 * Set the initial check for the XPDayAlarm.  We want it to go off on the
 * hour so the first check has to test for how long till that is.  Then we
 * set an hourly alarm.
 *
 *	Notes:
 *========================================================================*/
int
XPApptInitDayCheck()
{
	int			delay = 0;
	time_t		time_tp;
	struct tm	*tm_tp;

	time(&time_tp);
	tm_tp = localtime(&time_tp);
	delay = (60 - tm_tp->tm_min) * 60 * 1000;
	gtk_timeout_add (delay, (GtkFunction)DayCheckOne, NULL);
	return(FALSE);
}
int
DayCheckOne()
{
	int	appt_day_timer;

	ApptDayCheck();
	appt_day_timer = gtk_timeout_add (60*60*1000, 
		(GtkFunction)ApptDayCheck, NULL);
	return(FALSE);
}

/*========================================================================
 *	Name:			ApptDayCheck
 *	Prototype:	void ApptDayCheck()
 *					
 *	Description:
 * Check if the day has changed since we started the Calendar Manager.
 *
 *	Notes:
 *========================================================================*/
int
ApptDayCheck()
{
	GDate			*today = NULL;
	time_t		time_tp;
	struct tm	*tm_tp;

	time(&time_tp);
	tm_tp = localtime(&time_tp);

	/* Don't do this if the calendar hasn't been opened yet. */
	if ( appt_widget == NULL )
		return(TRUE);

	/* Get Today's date */
	today = 
		g_date_new_dmy( tm_tp->tm_mday, tm_tp->tm_mon+1, tm_tp->tm_year+1900);

	/* Compare it with what the currently set date is. */
	if ( g_date_compare(absolutedate, today) != 0 )
	{
		/* Today is a new day - tell the Calendar Manager that. */
		g_date_free(absolutedate);
		absolutedate = 
			g_date_new_dmy( tm_tp->tm_mday, tm_tp->tm_mon+1, tm_tp->tm_year+1900);
		SetUpMonthlyPage(tm_tp->tm_mon, tm_tp->tm_year+1900);
	}
	return(TRUE);
}

